import asyncio
import random
from http import HTTPStatus
from time import sleep
from typing import Any, Dict, Union

import httpx

from ... import errors
from ...client import AuthenticatedClient, Client
from ...models.background_task_status import BackgroundTaskStatus
from ...models.background_task_type import BackgroundTaskType
from ...models.http_error import HttpError
from ...models.http_validation_error import HTTPValidationError
from ...models.list_response_background_task_out import ListResponseBackgroundTaskOut
from ...models.ordering import Ordering
from ...types import UNSET, Response, Unset


def _get_kwargs(
    *,
    client: AuthenticatedClient,
    status: Union[Unset, None, BackgroundTaskStatus] = UNSET,
    task: Union[Unset, None, BackgroundTaskType] = UNSET,
    limit: Union[Unset, None, int] = UNSET,
    iterator: Union[Unset, None, str] = UNSET,
    order: Union[Unset, None, Ordering] = UNSET,
) -> Dict[str, Any]:
    url = "{}/api/v1/background-task".format(client.base_url)

    headers: Dict[str, str] = client.get_headers()
    cookies: Dict[str, Any] = client.get_cookies()

    params: Dict[str, Any] = {}
    json_status: Union[Unset, None, str] = UNSET
    if not isinstance(status, Unset):
        json_status = status.value if status else None

    params["status"] = json_status

    json_task: Union[Unset, None, str] = UNSET
    if not isinstance(task, Unset):
        json_task = task.value if task else None

    params["task"] = json_task

    params["limit"] = limit

    params["iterator"] = iterator

    json_order: Union[Unset, None, str] = UNSET
    if not isinstance(order, Unset):
        json_order = order.value if order else None

    params["order"] = json_order

    params = {k: v for k, v in params.items() if v is not UNSET and v is not None}

    return {
        "method": "get",
        "url": url,
        "headers": headers,
        "cookies": cookies,
        "timeout": client.get_timeout(),
        "follow_redirects": client.follow_redirects,
        "params": params,
    }


def _parse_response(*, client: Client, response: httpx.Response) -> ListResponseBackgroundTaskOut:
    if response.status_code == HTTPStatus.OK:
        response_200 = ListResponseBackgroundTaskOut.from_dict(response.json())

        return response_200
    if response.status_code == HTTPStatus.BAD_REQUEST:
        raise HttpError.init_exception(response.json(), response.status_code)
    if response.status_code == HTTPStatus.UNAUTHORIZED:
        raise HttpError.init_exception(response.json(), response.status_code)
    if response.status_code == HTTPStatus.FORBIDDEN:
        raise HttpError.init_exception(response.json(), response.status_code)
    if response.status_code == HTTPStatus.NOT_FOUND:
        raise HttpError.init_exception(response.json(), response.status_code)
    if response.status_code == HTTPStatus.CONFLICT:
        raise HttpError.init_exception(response.json(), response.status_code)
    if response.status_code == HTTPStatus.UNPROCESSABLE_ENTITY:
        raise HTTPValidationError.init_exception(response.json(), response.status_code)
    if response.status_code == HTTPStatus.TOO_MANY_REQUESTS:
        raise HttpError.init_exception(response.json(), response.status_code)
    raise errors.UnexpectedStatus(response.status_code, response.content)


def _build_response(*, client: Client, response: httpx.Response) -> Response[ListResponseBackgroundTaskOut]:
    return Response(
        status_code=HTTPStatus(response.status_code),
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(client=client, response=response),
    )


def request_sync_detailed(
    *,
    client: AuthenticatedClient,
    status: Union[Unset, None, BackgroundTaskStatus] = UNSET,
    task: Union[Unset, None, BackgroundTaskType] = UNSET,
    limit: Union[Unset, None, int] = UNSET,
    iterator: Union[Unset, None, str] = UNSET,
    order: Union[Unset, None, Ordering] = UNSET,
) -> Response[ListResponseBackgroundTaskOut]:
    """List Background Tasks

     List background tasks executed in the past 90 days.

    Args:
        status (Union[Unset, None, BackgroundTaskStatus]):
        task (Union[Unset, None, BackgroundTaskType]):
        limit (Union[Unset, None, int]): Limit the number of returned items
        iterator (Union[Unset, None, str]): The iterator returned from a prior invocation
        order (Union[Unset, None, Ordering]): Defines the ordering in a listing of results.

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[ListResponseBackgroundTaskOut]
    """

    kwargs = _get_kwargs(
        client=client,
        status=status,
        task=task,
        limit=limit,
        iterator=iterator,
        order=order,
    )

    kwargs["headers"] = {"svix-req-id": f"{random.getrandbits(32)}", **kwargs["headers"]}

    response = httpx.request(
        verify=client.verify_ssl,
        **kwargs,
    )
    for retry_count, sleep_time in enumerate(client.retry_schedule):
        if response.status_code < 500:
            break

        sleep(sleep_time)
        kwargs["headers"]["svix-retry-count"] = str(retry_count)
        response = httpx.request(
            verify=client.verify_ssl,
            **kwargs,
        )

    return _build_response(client=client, response=response)


def request_sync(
    *,
    client: AuthenticatedClient,
    status: Union[Unset, None, BackgroundTaskStatus] = UNSET,
    task: Union[Unset, None, BackgroundTaskType] = UNSET,
    limit: Union[Unset, None, int] = UNSET,
    iterator: Union[Unset, None, str] = UNSET,
    order: Union[Unset, None, Ordering] = UNSET,
) -> ListResponseBackgroundTaskOut:
    """List Background Tasks

     List background tasks executed in the past 90 days.

    Args:
        status (Union[Unset, None, BackgroundTaskStatus]):
        task (Union[Unset, None, BackgroundTaskType]):
        limit (Union[Unset, None, int]): Limit the number of returned items
        iterator (Union[Unset, None, str]): The iterator returned from a prior invocation
        order (Union[Unset, None, Ordering]): Defines the ordering in a listing of results.

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        ListResponseBackgroundTaskOut
    """

    return request_sync_detailed(
        client=client,
        status=status,
        task=task,
        limit=limit,
        iterator=iterator,
        order=order,
    ).parsed


async def request_asyncio_detailed(
    *,
    client: AuthenticatedClient,
    status: Union[Unset, None, BackgroundTaskStatus] = UNSET,
    task: Union[Unset, None, BackgroundTaskType] = UNSET,
    limit: Union[Unset, None, int] = UNSET,
    iterator: Union[Unset, None, str] = UNSET,
    order: Union[Unset, None, Ordering] = UNSET,
) -> Response[ListResponseBackgroundTaskOut]:
    """List Background Tasks

     List background tasks executed in the past 90 days.

    Args:
        status (Union[Unset, None, BackgroundTaskStatus]):
        task (Union[Unset, None, BackgroundTaskType]):
        limit (Union[Unset, None, int]): Limit the number of returned items
        iterator (Union[Unset, None, str]): The iterator returned from a prior invocation
        order (Union[Unset, None, Ordering]): Defines the ordering in a listing of results.

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[ListResponseBackgroundTaskOut]
    """

    kwargs = _get_kwargs(
        client=client,
        status=status,
        task=task,
        limit=limit,
        iterator=iterator,
        order=order,
    )

    kwargs["headers"] = {"svix-req-id": f"{random.getrandbits(32)}", **kwargs["headers"]}

    async with httpx.AsyncClient(verify=client.verify_ssl) as _client:
        response = await _client.request(**kwargs)

        for retry_count, sleep_time in enumerate(client.retry_schedule):
            if response.status_code < 500:
                break

            await asyncio.sleep(sleep_time)
            kwargs["headers"]["svix-retry-count"] = str(retry_count)
            response = await _client.request(**kwargs)

    return _build_response(client=client, response=response)


async def request_asyncio(
    *,
    client: AuthenticatedClient,
    status: Union[Unset, None, BackgroundTaskStatus] = UNSET,
    task: Union[Unset, None, BackgroundTaskType] = UNSET,
    limit: Union[Unset, None, int] = UNSET,
    iterator: Union[Unset, None, str] = UNSET,
    order: Union[Unset, None, Ordering] = UNSET,
) -> ListResponseBackgroundTaskOut:
    """List Background Tasks

     List background tasks executed in the past 90 days.

    Args:
        status (Union[Unset, None, BackgroundTaskStatus]):
        task (Union[Unset, None, BackgroundTaskType]):
        limit (Union[Unset, None, int]): Limit the number of returned items
        iterator (Union[Unset, None, str]): The iterator returned from a prior invocation
        order (Union[Unset, None, Ordering]): Defines the ordering in a listing of results.

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        ListResponseBackgroundTaskOut
    """

    return (
        await request_asyncio_detailed(
            client=client,
            status=status,
            task=task,
            limit=limit,
            iterator=iterator,
            order=order,
        )
    ).parsed



import asyncio
import datetime
import random
from http import HTTPStatus
from time import sleep
from typing import Any, Dict, List, Union

import httpx

from ... import errors
from ...client import AuthenticatedClient, Client
from ...models.http_error import HttpError
from ...models.http_validation_error import HTTPValidationError
from ...models.message_events_out import MessageEventsOut
from ...types import UNSET, Response, Unset


def _get_kwargs(
    *,
    client: AuthenticatedClient,
    limit: Union[Unset, None, int] = UNSET,
    iterator: Union[Unset, None, str] = UNSET,
    event_types: Union[Unset, None, List[str]] = UNSET,
    channels: Union[Unset, None, List[str]] = UNSET,
    after: Union[Unset, None, datetime.datetime] = UNSET,
) -> Dict[str, Any]:
    url = "{}/api/v1/events".format(client.base_url)

    headers: Dict[str, str] = client.get_headers()
    cookies: Dict[str, Any] = client.get_cookies()

    params: Dict[str, Any] = {}
    params["limit"] = limit

    params["iterator"] = iterator

    json_event_types: Union[Unset, None, List[str]] = UNSET
    if not isinstance(event_types, Unset):
        if event_types is None:
            json_event_types = None
        else:
            json_event_types = event_types

    params["event_types"] = json_event_types

    json_channels: Union[Unset, None, List[str]] = UNSET
    if not isinstance(channels, Unset):
        if channels is None:
            json_channels = None
        else:
            json_channels = channels

    params["channels"] = json_channels

    json_after: Union[Unset, None, str] = UNSET
    if not isinstance(after, Unset):
        json_after = after.isoformat() if after else None

    params["after"] = json_after

    params = {k: v for k, v in params.items() if v is not UNSET and v is not None}

    return {
        "method": "get",
        "url": url,
        "headers": headers,
        "cookies": cookies,
        "timeout": client.get_timeout(),
        "follow_redirects": client.follow_redirects,
        "params": params,
    }


def _parse_response(*, client: Client, response: httpx.Response) -> MessageEventsOut:
    if response.status_code == HTTPStatus.OK:
        response_200 = MessageEventsOut.from_dict(response.json())

        return response_200
    if response.status_code == HTTPStatus.BAD_REQUEST:
        raise HttpError.init_exception(response.json(), response.status_code)
    if response.status_code == HTTPStatus.UNAUTHORIZED:
        raise HttpError.init_exception(response.json(), response.status_code)
    if response.status_code == HTTPStatus.FORBIDDEN:
        raise HttpError.init_exception(response.json(), response.status_code)
    if response.status_code == HTTPStatus.NOT_FOUND:
        raise HttpError.init_exception(response.json(), response.status_code)
    if response.status_code == HTTPStatus.CONFLICT:
        raise HttpError.init_exception(response.json(), response.status_code)
    if response.status_code == HTTPStatus.UNPROCESSABLE_ENTITY:
        raise HTTPValidationError.init_exception(response.json(), response.status_code)
    if response.status_code == HTTPStatus.TOO_MANY_REQUESTS:
        raise HttpError.init_exception(response.json(), response.status_code)
    raise errors.UnexpectedStatus(response.status_code, response.content)


def _build_response(*, client: Client, response: httpx.Response) -> Response[MessageEventsOut]:
    return Response(
        status_code=HTTPStatus(response.status_code),
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(client=client, response=response),
    )


def request_sync_detailed(
    *,
    client: AuthenticatedClient,
    limit: Union[Unset, None, int] = UNSET,
    iterator: Union[Unset, None, str] = UNSET,
    event_types: Union[Unset, None, List[str]] = UNSET,
    channels: Union[Unset, None, List[str]] = UNSET,
    after: Union[Unset, None, datetime.datetime] = UNSET,
) -> Response[MessageEventsOut]:
    """Events

     Reads the stream of operational webhook events for this environment

    Args:
        limit (Union[Unset, None, int]): Limit the number of returned items
        iterator (Union[Unset, None, str]): The iterator returned from a prior invocation
        event_types (Union[Unset, None, List[str]]): Filter response based on the event type
        channels (Union[Unset, None, List[str]]): Filter response based on the event type
        after (Union[Unset, None, datetime.datetime]):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[MessageEventsOut]
    """

    kwargs = _get_kwargs(
        client=client,
        limit=limit,
        iterator=iterator,
        event_types=event_types,
        channels=channels,
        after=after,
    )

    kwargs["headers"] = {"svix-req-id": f"{random.getrandbits(32)}", **kwargs["headers"]}

    response = httpx.request(
        verify=client.verify_ssl,
        **kwargs,
    )
    for retry_count, sleep_time in enumerate(client.retry_schedule):
        if response.status_code < 500:
            break

        sleep(sleep_time)
        kwargs["headers"]["svix-retry-count"] = str(retry_count)
        response = httpx.request(
            verify=client.verify_ssl,
            **kwargs,
        )

    return _build_response(client=client, response=response)


def request_sync(
    *,
    client: AuthenticatedClient,
    limit: Union[Unset, None, int] = UNSET,
    iterator: Union[Unset, None, str] = UNSET,
    event_types: Union[Unset, None, List[str]] = UNSET,
    channels: Union[Unset, None, List[str]] = UNSET,
    after: Union[Unset, None, datetime.datetime] = UNSET,
) -> MessageEventsOut:
    """Events

     Reads the stream of operational webhook events for this environment

    Args:
        limit (Union[Unset, None, int]): Limit the number of returned items
        iterator (Union[Unset, None, str]): The iterator returned from a prior invocation
        event_types (Union[Unset, None, List[str]]): Filter response based on the event type
        channels (Union[Unset, None, List[str]]): Filter response based on the event type
        after (Union[Unset, None, datetime.datetime]):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        MessageEventsOut
    """

    return request_sync_detailed(
        client=client,
        limit=limit,
        iterator=iterator,
        event_types=event_types,
        channels=channels,
        after=after,
    ).parsed


async def request_asyncio_detailed(
    *,
    client: AuthenticatedClient,
    limit: Union[Unset, None, int] = UNSET,
    iterator: Union[Unset, None, str] = UNSET,
    event_types: Union[Unset, None, List[str]] = UNSET,
    channels: Union[Unset, None, List[str]] = UNSET,
    after: Union[Unset, None, datetime.datetime] = UNSET,
) -> Response[MessageEventsOut]:
    """Events

     Reads the stream of operational webhook events for this environment

    Args:
        limit (Union[Unset, None, int]): Limit the number of returned items
        iterator (Union[Unset, None, str]): The iterator returned from a prior invocation
        event_types (Union[Unset, None, List[str]]): Filter response based on the event type
        channels (Union[Unset, None, List[str]]): Filter response based on the event type
        after (Union[Unset, None, datetime.datetime]):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[MessageEventsOut]
    """

    kwargs = _get_kwargs(
        client=client,
        limit=limit,
        iterator=iterator,
        event_types=event_types,
        channels=channels,
        after=after,
    )

    kwargs["headers"] = {"svix-req-id": f"{random.getrandbits(32)}", **kwargs["headers"]}

    async with httpx.AsyncClient(verify=client.verify_ssl) as _client:
        response = await _client.request(**kwargs)

        for retry_count, sleep_time in enumerate(client.retry_schedule):
            if response.status_code < 500:
                break

            await asyncio.sleep(sleep_time)
            kwargs["headers"]["svix-retry-count"] = str(retry_count)
            response = await _client.request(**kwargs)

    return _build_response(client=client, response=response)


async def request_asyncio(
    *,
    client: AuthenticatedClient,
    limit: Union[Unset, None, int] = UNSET,
    iterator: Union[Unset, None, str] = UNSET,
    event_types: Union[Unset, None, List[str]] = UNSET,
    channels: Union[Unset, None, List[str]] = UNSET,
    after: Union[Unset, None, datetime.datetime] = UNSET,
) -> MessageEventsOut:
    """Events

     Reads the stream of operational webhook events for this environment

    Args:
        limit (Union[Unset, None, int]): Limit the number of returned items
        iterator (Union[Unset, None, str]): The iterator returned from a prior invocation
        event_types (Union[Unset, None, List[str]]): Filter response based on the event type
        channels (Union[Unset, None, List[str]]): Filter response based on the event type
        after (Union[Unset, None, datetime.datetime]):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        MessageEventsOut
    """

    return (
        await request_asyncio_detailed(
            client=client,
            limit=limit,
            iterator=iterator,
            event_types=event_types,
            channels=channels,
            after=after,
        )
    ).parsed



# TODO - remove this special case when we fix the generated code for empty openapi structs
from typing import Any, Dict, List, Type, TypeVar

import attr

T = TypeVar("T", bound="S3Config")


@attr.s(auto_attribs=True)
class S3Config:
    """
    Attributes:
        access_key_id (str):
        bucket (str):
        region (str):
        secret_access_key (str):
    """

    access_key_id: str
    bucket: str
    region: str
    secret_access_key: str
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        access_key_id = self.access_key_id
        bucket = self.bucket
        region = self.region
        secret_access_key = self.secret_access_key

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "accessKeyId": access_key_id,
                "bucket": bucket,
                "region": region,
                "secretAccessKey": secret_access_key,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        access_key_id = d.pop("accessKeyId")

        bucket = d.pop("bucket")

        region = d.pop("region")

        secret_access_key = d.pop("secretAccessKey")

        s3_config = cls(
            access_key_id=access_key_id,
            bucket=bucket,
            region=region,
            secret_access_key=secret_access_key,
        )

        s3_config.additional_properties = d
        return s3_config

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



# TODO - remove this special case when we fix the generated code for empty openapi structs
from typing import Any, Dict, List, Type, TypeVar

import attr

T = TypeVar("T", bound="GoogleCloudStorageConfig")


@attr.s(auto_attribs=True)
class GoogleCloudStorageConfig:
    """Configuration for a Google Cloud Storage sink.

    Write stream events into the named bucket using the supplied Google Cloud credentials.

        Attributes:
            bucket (str):
            credentials (str): Google Cloud Credentials JSON Object as a string.
    """

    bucket: str
    credentials: str
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        bucket = self.bucket
        credentials = self.credentials

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "bucket": bucket,
                "credentials": credentials,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        bucket = d.pop("bucket")

        credentials = d.pop("credentials")

        google_cloud_storage_config = cls(
            bucket=bucket,
            credentials=credentials,
        )

        google_cloud_storage_config.additional_properties = d
        return google_cloud_storage_config

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



# TODO - remove this special case when we fix the generated code for empty openapi structs
from typing import Any, Dict, List, Type, TypeVar

import attr

T = TypeVar("T", bound="InboundPathParams")


@attr.s(auto_attribs=True)
class InboundPathParams:
    """
    Attributes:
        app_id (str): The app's ID or UID Example: unique-app-identifier.
        inbound_token (str):
    """

    app_id: str
    inbound_token: str
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        app_id = self.app_id
        inbound_token = self.inbound_token

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "app_id": app_id,
                "inbound_token": inbound_token,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        app_id = d.pop("app_id")

        inbound_token = d.pop("inbound_token")

        inbound_path_params = cls(
            app_id=app_id,
            inbound_token=inbound_token,
        )

        inbound_path_params.additional_properties = d
        return inbound_path_params

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from enum import Enum


class Oauth2GrantTypeIn(str, Enum):
    CLIENTCREDENTIALS = "clientCredentials"
    REFRESHTOKEN = "refreshToken"

    def __str__(self) -> str:
        return str(self.value)



# TODO - remove this special case when we fix the generated code for empty openapi structs
from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union

import attr

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.event_type_from_open_api_schemas import EventTypeFromOpenApiSchemas


T = TypeVar("T", bound="EventTypeFromOpenApi")


@attr.s(auto_attribs=True)
class EventTypeFromOpenApi:
    """
    Attributes:
        deprecated (bool):
        description (str):
        name (str): The event type's name Example: user.signup.
        schemas (Union[Unset, None, EventTypeFromOpenApiSchemas]):  Example: {'description': 'An invoice was paid by a
            user', 'properties': {'invoiceId': {'description': 'The invoice id', 'type': 'string'}, 'userId':
            {'description': 'The user id', 'type': 'string'}}, 'required': ['invoiceId', 'userId'], 'title': 'Invoice Paid
            Event', 'type': 'object'}.
    """

    deprecated: bool
    description: str
    name: str
    schemas: Union[Unset, None, "EventTypeFromOpenApiSchemas"] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        deprecated = self.deprecated
        description = self.description
        name = self.name
        schemas = self.schemas

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "deprecated": deprecated,
                "description": description,
                "name": name,
            }
        )
        if schemas is not UNSET:
            field_dict["schemas"] = schemas

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        deprecated = d.pop("deprecated")

        description = d.pop("description")

        name = d.pop("name")

        schemas = d.pop("schemas", UNSET)

        event_type_from_open_api = cls(
            deprecated=deprecated,
            description=description,
            name=name,
            schemas=schemas,
        )

        event_type_from_open_api.additional_properties = d
        return event_type_from_open_api

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



# TODO - remove this special case when we fix the generated code for empty openapi structs
from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar

import attr

from ..models.stream_sink_patch_type_3_type import StreamSinkPatchType3Type

if TYPE_CHECKING:
    from ..models.s3_config import S3Config


T = TypeVar("T", bound="StreamSinkPatchType3")


@attr.s(auto_attribs=True)
class StreamSinkPatchType3:
    """
    Attributes:
        config (S3Config):
        type (StreamSinkPatchType3Type):
    """

    config: "S3Config"
    type: StreamSinkPatchType3Type
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        config = self.config.to_dict()

        type = self.type.value

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "config": config,
                "type": type,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.s3_config import S3Config

        d = src_dict.copy()
        config = S3Config.from_dict(d.pop("config"))

        type = StreamSinkPatchType3Type(d.pop("type"))

        stream_sink_patch_type_3 = cls(
            config=config,
            type=type,
        )

        stream_sink_patch_type_3.additional_properties = d
        return stream_sink_patch_type_3

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



# TODO - remove this special case when we fix the generated code for empty openapi structs
from typing import Any, Dict, List, Type, TypeVar, Union

import attr

from ..models.oauth_jws_signing_algorithm import OauthJwsSigningAlgorithm
from ..types import UNSET, Unset

T = TypeVar("T", bound="ClientSecretJwtParamsIn")


@attr.s(auto_attribs=True)
class ClientSecretJwtParamsIn:
    """
    Attributes:
        secret_base_64 (str): The base64-encoded secret used for signing the JWT.
        signing_algorithm (OauthJwsSigningAlgorithm):
        secret_id (Union[Unset, None, str]): Optional secret identifier. If supplied, this will be populated in the JWT
            header in the `kid` field.
        token_expiry_secs (Union[Unset, None, int]): Optional number of seconds after which the JWT should expire.
            Defaults to 300 seconds.
    """

    secret_base_64: str
    signing_algorithm: OauthJwsSigningAlgorithm
    secret_id: Union[Unset, None, str] = UNSET
    token_expiry_secs: Union[Unset, None, int] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        secret_base_64 = self.secret_base_64
        signing_algorithm = self.signing_algorithm.value

        secret_id = self.secret_id
        token_expiry_secs = self.token_expiry_secs

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "secretBase64": secret_base_64,
                "signingAlgorithm": signing_algorithm,
            }
        )
        if secret_id is not UNSET:
            field_dict["secretId"] = secret_id
        if token_expiry_secs is not UNSET:
            field_dict["tokenExpirySecs"] = token_expiry_secs

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        secret_base_64 = d.pop("secretBase64")

        signing_algorithm = OauthJwsSigningAlgorithm(d.pop("signingAlgorithm"))

        secret_id = d.pop("secretId", UNSET)

        token_expiry_secs = d.pop("tokenExpirySecs", UNSET)

        client_secret_jwt_params_in = cls(
            secret_base_64=secret_base_64,
            signing_algorithm=signing_algorithm,
            secret_id=secret_id,
            token_expiry_secs=token_expiry_secs,
        )

        client_secret_jwt_params_in.additional_properties = d
        return client_secret_jwt_params_in

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



# TODO - remove this special case when we fix the generated code for empty openapi structs
import datetime
from typing import Any, Dict, List, Type, TypeVar, Union

import attr
from dateutil.parser import isoparse

from ..types import UNSET, Unset

T = TypeVar("T", bound="EndpointDisabledEventData")


@attr.s(auto_attribs=True)
class EndpointDisabledEventData:
    """Sent when an endpoint has been automatically disabled after continuous failures.

    Attributes:
        app_id (str): The app's ID Example: app_1srOrx2ZWZBpBUvZwXKQmoEYga2.
        endpoint_id (str): The ep's ID Example: ep_1srOrx2ZWZBpBUvZwXKQmoEYga2.
        fail_since (datetime.datetime):
        app_uid (Union[Unset, None, str]): The app's UID Example: unique-app-identifier.
        endpoint_uid (Union[Unset, None, str]): The ep's UID Example: unique-ep-identifier.
    """

    app_id: str
    endpoint_id: str
    fail_since: datetime.datetime
    app_uid: Union[Unset, None, str] = UNSET
    endpoint_uid: Union[Unset, None, str] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        app_id = self.app_id
        endpoint_id = self.endpoint_id
        fail_since = self.fail_since.isoformat()

        app_uid = self.app_uid
        endpoint_uid = self.endpoint_uid

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "appId": app_id,
                "endpointId": endpoint_id,
                "failSince": fail_since,
            }
        )
        if app_uid is not UNSET:
            field_dict["appUid"] = app_uid
        if endpoint_uid is not UNSET:
            field_dict["endpointUid"] = endpoint_uid

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        app_id = d.pop("appId")

        endpoint_id = d.pop("endpointId")

        fail_since = isoparse(d.pop("failSince"))

        app_uid = d.pop("appUid", UNSET)

        endpoint_uid = d.pop("endpointUid", UNSET)

        endpoint_disabled_event_data = cls(
            app_id=app_id,
            endpoint_id=endpoint_id,
            fail_since=fail_since,
            app_uid=app_uid,
            endpoint_uid=endpoint_uid,
        )

        endpoint_disabled_event_data.additional_properties = d
        return endpoint_disabled_event_data

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



# TODO - remove this special case when we fix the generated code for empty openapi structs
from typing import Any, Dict, List, Type, TypeVar, Union

import attr

from ..models.kafka_security_protocol_type import KafkaSecurityProtocolType
from ..models.sink_out_type_2_type import SinkOutType2Type
from ..types import UNSET, Unset

T = TypeVar("T", bound="SinkOutType2")


@attr.s(auto_attribs=True)
class SinkOutType2:
    """
    Attributes:
        brokers (str):
        security_protocol (KafkaSecurityProtocolType):
        topic (str):
        type (SinkOutType2Type):
        sasl_password (Union[Unset, None, str]): Password for SASL, if `security_protocol` is `sasl-ssl`.
        sasl_username (Union[Unset, None, str]): Username for SASL, if `security_protocol` is `sasl-ssl`.
    """

    brokers: str
    security_protocol: KafkaSecurityProtocolType
    topic: str
    type: SinkOutType2Type
    sasl_password: Union[Unset, None, str] = UNSET
    sasl_username: Union[Unset, None, str] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        brokers = self.brokers
        security_protocol = self.security_protocol.value

        topic = self.topic
        type = self.type.value

        sasl_password = self.sasl_password
        sasl_username = self.sasl_username

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "brokers": brokers,
                "securityProtocol": security_protocol,
                "topic": topic,
                "type": type,
            }
        )
        if sasl_password is not UNSET:
            field_dict["saslPassword"] = sasl_password
        if sasl_username is not UNSET:
            field_dict["saslUsername"] = sasl_username

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        brokers = d.pop("brokers")

        security_protocol = KafkaSecurityProtocolType(d.pop("securityProtocol"))

        topic = d.pop("topic")

        type = SinkOutType2Type(d.pop("type"))

        sasl_password = d.pop("saslPassword", UNSET)

        sasl_username = d.pop("saslUsername", UNSET)

        sink_out_type_2 = cls(
            brokers=brokers,
            security_protocol=security_protocol,
            topic=topic,
            type=type,
            sasl_password=sasl_password,
            sasl_username=sasl_username,
        )

        sink_out_type_2.additional_properties = d
        return sink_out_type_2

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



# TODO - remove this special case when we fix the generated code for empty openapi structs
from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar

import attr

if TYPE_CHECKING:
    from ..models.completion_choice import CompletionChoice


T = TypeVar("T", bound="GenerateOut")


@attr.s(auto_attribs=True)
class GenerateOut:
    """
    Attributes:
        choices (List['CompletionChoice']):
        created (int):
        id (str):
        model (str):
        object_ (str):
    """

    choices: List["CompletionChoice"]
    created: int
    id: str
    model: str
    object_: str
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        choices = []
        for choices_item_data in self.choices:
            choices_item = choices_item_data.to_dict()

            choices.append(choices_item)

        created = self.created
        id = self.id
        model = self.model
        object_ = self.object_

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "choices": choices,
                "created": created,
                "id": id,
                "model": model,
                "object": object_,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.completion_choice import CompletionChoice

        d = src_dict.copy()
        choices = []
        _choices = d.pop("choices")
        for choices_item_data in _choices:
            choices_item = CompletionChoice.from_dict(choices_item_data)

            choices.append(choices_item)

        created = d.pop("created")

        id = d.pop("id")

        model = d.pop("model")

        object_ = d.pop("object")

        generate_out = cls(
            choices=choices,
            created=created,
            id=id,
            model=model,
            object_=object_,
        )

        generate_out.additional_properties = d
        return generate_out

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



# TODO - remove this special case when we fix the generated code for empty openapi structs
from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union, cast

import attr

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.application_in import ApplicationIn


T = TypeVar("T", bound="AppPortalAccessIn")


@attr.s(auto_attribs=True)
class AppPortalAccessIn:
    """
    Attributes:
        application (Union[Unset, ApplicationIn]):
        expiry (Union[Unset, None, int]): How long the token will be valid for, in seconds.

            Valid values are between 1 hour and 7 days. The default is 7 days. Default: 604800.
        feature_flags (Union[Unset, List[str]]): The set of feature flags the created token will have access to.
        read_only (Union[Unset, None, bool]): Whether the app portal should be in read-only mode.
    """

    application: Union[Unset, "ApplicationIn"] = UNSET
    expiry: Union[Unset, None, int] = 604800
    feature_flags: Union[Unset, List[str]] = UNSET
    read_only: Union[Unset, None, bool] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        application: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.application, Unset):
            application = self.application.to_dict()

        expiry = self.expiry
        feature_flags: Union[Unset, List[str]] = UNSET
        if not isinstance(self.feature_flags, Unset):
            feature_flags = self.feature_flags

        read_only = self.read_only

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if application is not UNSET:
            field_dict["application"] = application
        if expiry is not UNSET:
            field_dict["expiry"] = expiry
        if feature_flags is not UNSET:
            field_dict["featureFlags"] = feature_flags
        if read_only is not UNSET:
            field_dict["readOnly"] = read_only

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.application_in import ApplicationIn

        d = src_dict.copy()
        _application = d.pop("application", UNSET)
        application: Union[Unset, ApplicationIn]
        if isinstance(_application, Unset):
            application = UNSET
        else:
            application = ApplicationIn.from_dict(_application)

        expiry = d.pop("expiry", UNSET)

        feature_flags = cast(List[str], d.pop("featureFlags", UNSET))

        read_only = d.pop("readOnly", UNSET)

        app_portal_access_in = cls(
            application=application,
            expiry=expiry,
            feature_flags=feature_flags,
            read_only=read_only,
        )

        app_portal_access_in.additional_properties = d
        return app_portal_access_in

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



# TODO - remove this special case when we fix the generated code for empty openapi structs
from typing import Any, Dict, List, Type, TypeVar

import attr

from ..models.sink_out_type_1_type import SinkOutType1Type

T = TypeVar("T", bound="SinkOutType1")


@attr.s(auto_attribs=True)
class SinkOutType1:
    """
    Attributes:
        access_key (str):
        queue_dsn (str):
        region (str):
        secret_key (str):
        type (SinkOutType1Type):
    """

    access_key: str
    queue_dsn: str
    region: str
    secret_key: str
    type: SinkOutType1Type
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        access_key = self.access_key
        queue_dsn = self.queue_dsn
        region = self.region
        secret_key = self.secret_key
        type = self.type.value

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "accessKey": access_key,
                "queueDsn": queue_dsn,
                "region": region,
                "secretKey": secret_key,
                "type": type,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        access_key = d.pop("accessKey")

        queue_dsn = d.pop("queueDsn")

        region = d.pop("region")

        secret_key = d.pop("secretKey")

        type = SinkOutType1Type(d.pop("type"))

        sink_out_type_1 = cls(
            access_key=access_key,
            queue_dsn=queue_dsn,
            region=region,
            secret_key=secret_key,
            type=type,
        )

        sink_out_type_1.additional_properties = d
        return sink_out_type_1

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



# TODO - remove this special case when we fix the generated code for empty openapi structs
import datetime
from typing import Any, Dict, List, Type, TypeVar, Union, cast

import attr
from dateutil.parser import isoparse

from ..models.message_status import MessageStatus
from ..types import UNSET, Unset

T = TypeVar("T", bound="MessageEndpointOut")


@attr.s(auto_attribs=True)
class MessageEndpointOut:
    """
    Attributes:
        created_at (datetime.datetime):
        description (str): An example endpoint name
        id (str): The ep's ID Example: ep_1srOrx2ZWZBpBUvZwXKQmoEYga2.
        status (MessageStatus): The sending status of the message:
            - Success = 0
            - Pending = 1
            - Fail = 2
            - Sending = 3
        updated_at (datetime.datetime):
        url (str):  Example: https://example.com/webhook/.
        version (int):  Example: 1.
        channels (Union[Unset, None, List[str]]): List of message channels this endpoint listens to (omit for all)
            Example: ['project_123', 'group_2'].
        disabled (Union[Unset, bool]):
        filter_types (Union[Unset, None, List[str]]):  Example: ['user.signup', 'user.deleted'].
        next_attempt (Union[Unset, None, datetime.datetime]):
        rate_limit (Union[Unset, None, int]):
        uid (Union[Unset, None, str]): Optional unique identifier for the endpoint Example: unique-ep-identifier.
    """

    created_at: datetime.datetime
    description: str
    id: str
    status: MessageStatus
    updated_at: datetime.datetime
    url: str
    version: int
    channels: Union[Unset, None, List[str]] = UNSET
    disabled: Union[Unset, bool] = False
    filter_types: Union[Unset, None, List[str]] = UNSET
    next_attempt: Union[Unset, None, datetime.datetime] = UNSET
    rate_limit: Union[Unset, None, int] = UNSET
    uid: Union[Unset, None, str] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        created_at = self.created_at.isoformat()

        description = self.description
        id = self.id
        status = self.status.value

        updated_at = self.updated_at.isoformat()

        url = self.url
        version = self.version
        channels: Union[Unset, None, List[str]] = UNSET
        if not isinstance(self.channels, Unset):
            if self.channels is None:
                channels = None
            else:
                channels = self.channels

        disabled = self.disabled
        filter_types: Union[Unset, None, List[str]] = UNSET
        if not isinstance(self.filter_types, Unset):
            if self.filter_types is None:
                filter_types = None
            else:
                filter_types = self.filter_types

        next_attempt: Union[Unset, None, str] = UNSET
        if not isinstance(self.next_attempt, Unset):
            next_attempt = self.next_attempt.isoformat() if self.next_attempt else None

        rate_limit = self.rate_limit
        uid = self.uid

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "createdAt": created_at,
                "description": description,
                "id": id,
                "status": status,
                "updatedAt": updated_at,
                "url": url,
                "version": version,
            }
        )
        if channels is not UNSET:
            field_dict["channels"] = channels
        if disabled is not UNSET:
            field_dict["disabled"] = disabled
        if filter_types is not UNSET:
            field_dict["filterTypes"] = filter_types
        if next_attempt is not UNSET:
            field_dict["nextAttempt"] = next_attempt
        if rate_limit is not UNSET:
            field_dict["rateLimit"] = rate_limit
        if uid is not UNSET:
            field_dict["uid"] = uid

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        created_at = isoparse(d.pop("createdAt"))

        description = d.pop("description")

        id = d.pop("id")

        status = MessageStatus(d.pop("status"))

        updated_at = isoparse(d.pop("updatedAt"))

        url = d.pop("url")

        version = d.pop("version")

        channels = cast(List[str], d.pop("channels", UNSET))

        disabled = d.pop("disabled", UNSET)

        filter_types = cast(List[str], d.pop("filterTypes", UNSET))

        _next_attempt = d.pop("nextAttempt", UNSET)
        next_attempt: Union[Unset, None, datetime.datetime]
        if _next_attempt is None:
            next_attempt = None
        elif isinstance(_next_attempt, Unset):
            next_attempt = UNSET
        else:
            next_attempt = isoparse(_next_attempt)

        rate_limit = d.pop("rateLimit", UNSET)

        uid = d.pop("uid", UNSET)

        message_endpoint_out = cls(
            created_at=created_at,
            description=description,
            id=id,
            status=status,
            updated_at=updated_at,
            url=url,
            version=version,
            channels=channels,
            disabled=disabled,
            filter_types=filter_types,
            next_attempt=next_attempt,
            rate_limit=rate_limit,
            uid=uid,
        )

        message_endpoint_out.additional_properties = d
        return message_endpoint_out

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



import typing as t
from dataclasses import asdict, dataclass, field
from datetime import datetime, timezone

from deprecated import deprecated

from svix.internal.openapi_client.models.aggregate_event_types_out import (
    AggregateEventTypesOut,
)
from svix.internal.openapi_client.models.app_usage_stats_in import AppUsageStatsIn
from svix.internal.openapi_client.models.app_usage_stats_out import AppUsageStatsOut

from .internal.openapi_client.api.application import (
    v1_application_create,
    v1_application_delete,
    v1_application_get,
    v1_application_list,
    v1_application_patch,
    v1_application_update,
)
from .internal.openapi_client.api.authentication import (
    v1_authentication_app_portal_access,
    v1_authentication_dashboard_access,
    v1_authentication_logout,
)
from .internal.openapi_client.api.background_tasks import (
    get_background_task,
    list_background_tasks,
)
from .internal.openapi_client.api.endpoint import (
    v1_endpoint_create,
    v1_endpoint_delete,
    v1_endpoint_get,
    v1_endpoint_get_headers,
    v1_endpoint_get_secret,
    v1_endpoint_get_stats,
    v1_endpoint_list,
    v1_endpoint_patch,
    v1_endpoint_patch_headers,
    v1_endpoint_recover,
    v1_endpoint_replay,
    v1_endpoint_rotate_secret,
    v1_endpoint_send_example,
    v1_endpoint_transformation_get,
    v1_endpoint_transformation_partial_update,
    v1_endpoint_update,
    v1_endpoint_update_headers,
)
from .internal.openapi_client.api.event_type import (
    v1_event_type_create,
    v1_event_type_delete,
    v1_event_type_get,
    v1_event_type_import_openapi,
    v1_event_type_list,
    v1_event_type_patch,
    v1_event_type_update,
)
from .internal.openapi_client.api.integration import (
    v1_integration_create,
    v1_integration_delete,
    v1_integration_get,
    v1_integration_get_key,
    v1_integration_list,
    v1_integration_rotate_key,
    v1_integration_update,
)
from .internal.openapi_client.api.message import (
    v1_message_create,
    v1_message_expunge_content,
    v1_message_get,
    v1_message_list,
)
from .internal.openapi_client.api.message_attempt import (
    v1_message_attempt_expunge_content,
    v1_message_attempt_get,
    v1_message_attempt_list_attempted_destinations,
    v1_message_attempt_list_attempted_messages,
    v1_message_attempt_list_by_endpoint,
    v1_message_attempt_list_by_endpoint_deprecated,
    v1_message_attempt_list_by_msg,
    v1_message_attempt_resend,
)
from .internal.openapi_client.api.statistics import (
    v1_statistics_aggregate_app_stats,
    v1_statistics_aggregate_event_types,
)
from .internal.openapi_client.client import AuthenticatedClient
from .internal.openapi_client.models.app_portal_access_in import AppPortalAccessIn
from .internal.openapi_client.models.app_portal_access_out import AppPortalAccessOut
from .internal.openapi_client.models.application_in import ApplicationIn
from .internal.openapi_client.models.application_out import ApplicationOut
from .internal.openapi_client.models.application_patch import ApplicationPatch
from .internal.openapi_client.models.background_task_out import BackgroundTaskOut
from .internal.openapi_client.models.background_task_status import BackgroundTaskStatus
from .internal.openapi_client.models.background_task_type import BackgroundTaskType
from .internal.openapi_client.models.dashboard_access_out import DashboardAccessOut
from .internal.openapi_client.models.endpoint_headers_in import EndpointHeadersIn
from .internal.openapi_client.models.endpoint_headers_out import EndpointHeadersOut
from .internal.openapi_client.models.endpoint_headers_patch_in import (
    EndpointHeadersPatchIn,
)
from .internal.openapi_client.models.endpoint_in import EndpointIn
from .internal.openapi_client.models.endpoint_message_out_payload import (
    EndpointMessageOutPayload,
)
from .internal.openapi_client.models.endpoint_out import EndpointOut
from .internal.openapi_client.models.endpoint_patch import EndpointPatch
from .internal.openapi_client.models.endpoint_secret_out import EndpointSecretOut
from .internal.openapi_client.models.endpoint_secret_rotate_in import (
    EndpointSecretRotateIn,
)
from .internal.openapi_client.models.endpoint_stats import EndpointStats
from .internal.openapi_client.models.endpoint_transformation_in import (
    EndpointTransformationIn,
)
from .internal.openapi_client.models.endpoint_transformation_out import (
    EndpointTransformationOut,
)
from .internal.openapi_client.models.endpoint_update import EndpointUpdate
from .internal.openapi_client.models.event_example_in import EventExampleIn
from .internal.openapi_client.models.event_type_import_open_api_in import (
    EventTypeImportOpenApiIn,
)
from .internal.openapi_client.models.event_type_import_open_api_out import (
    EventTypeImportOpenApiOut,
)
from .internal.openapi_client.models.event_type_in import EventTypeIn
from .internal.openapi_client.models.event_type_out import EventTypeOut
from .internal.openapi_client.models.event_type_patch import EventTypePatch
from .internal.openapi_client.models.event_type_update import EventTypeUpdate
from .internal.openapi_client.models.integration_in import IntegrationIn
from .internal.openapi_client.models.integration_key_out import IntegrationKeyOut
from .internal.openapi_client.models.integration_out import IntegrationOut
from .internal.openapi_client.models.integration_update import IntegrationUpdate
from .internal.openapi_client.models.list_response_application_out import (
    ListResponseApplicationOut,
)
from .internal.openapi_client.models.list_response_background_task_out import (
    ListResponseBackgroundTaskOut,
)
from .internal.openapi_client.models.list_response_endpoint_message_out import (
    ListResponseEndpointMessageOut,
)
from .internal.openapi_client.models.list_response_endpoint_out import (
    ListResponseEndpointOut,
)
from .internal.openapi_client.models.list_response_event_type_out import (
    ListResponseEventTypeOut,
)
from .internal.openapi_client.models.list_response_integration_out import (
    ListResponseIntegrationOut,
)
from .internal.openapi_client.models.list_response_message_attempt_endpoint_out import (
    ListResponseMessageAttemptEndpointOut,
)
from .internal.openapi_client.models.list_response_message_attempt_out import (
    ListResponseMessageAttemptOut,
)
from .internal.openapi_client.models.list_response_message_endpoint_out import (
    ListResponseMessageEndpointOut,
)
from .internal.openapi_client.models.list_response_message_out import (
    ListResponseMessageOut,
)
from .internal.openapi_client.models.message_attempt_out import MessageAttemptOut
from .internal.openapi_client.models.message_in import MessageIn
from .internal.openapi_client.models.message_in_payload import MessageInPayload
from .internal.openapi_client.models.message_out import MessageOut
from .internal.openapi_client.models.message_out_payload import MessageOutPayload
from .internal.openapi_client.models.message_status import MessageStatus
from .internal.openapi_client.models.ordering import Ordering
from .internal.openapi_client.models.recover_in import RecoverIn
from .internal.openapi_client.models.recover_out import RecoverOut
from .internal.openapi_client.models.replay_in import ReplayIn
from .internal.openapi_client.models.replay_out import ReplayOut
from .internal.openapi_client.models.status_code_class import StatusCodeClass

DEFAULT_SERVER_URL = "https://api.svix.com"


def ensure_tz(x: t.Optional[datetime]) -> t.Optional[datetime]:
    if x is None:
        return None

    if x.tzinfo is None:
        return x.replace(tzinfo=timezone.utc)
    return x


@dataclass
class SvixOptions:
    debug: bool = False
    server_url: t.Optional[str] = None
    """
    The retry schedule, as seconds to wait after each failed request.

    The first entry is the time in seconds to wait between the first request
    failing and the first retry, and so on.
    Up to five retries are supported, passing a retry schedule with more than
    five entries will raise a `ValueError`.

    Defaults to [0.05, 0.1, 0.2]
    """
    retry_schedule: t.List[float] = field(default_factory=lambda: [0.05, 0.1, 0.2])

    """
    The maximum amount of time in seconds a request can take.

    Request methods will raise httpx.TimeoutException if this is exceeded.
    """
    timeout: float = 15.0


@dataclass
class ListOptions:
    iterator: t.Optional[str] = None
    limit: t.Optional[int] = None

    def to_dict(self) -> t.Dict[str, t.Any]:
        return {k: v for k, v in asdict(self).items() if v is not None}


@dataclass
class PostOptions:
    idempotency_key: t.Optional[str] = None

    def to_dict(self) -> t.Dict[str, t.Any]:
        return {k: v for k, v in asdict(self).items() if v is not None}


@dataclass
class MessageListOptions(ListOptions):
    event_types: t.Optional[t.List[str]] = None
    before: t.Optional[datetime] = None
    after: t.Optional[datetime] = None
    channel: t.Optional[str] = None
    tag: t.Optional[str] = None

    def to_dict(self) -> t.Dict[str, t.Any]:
        d = super().to_dict()
        if self.before is not None:
            d["before"] = ensure_tz(self.before)
        if self.after is not None:
            d["after"] = ensure_tz(self.after)
        return d


@dataclass
class ApplicationListOptions(ListOptions):
    order: t.Optional[Ordering] = None


@dataclass
class EventTypeListOptions(ListOptions):
    with_content: t.Optional[bool] = None
    include_archived: t.Optional[bool] = None


@dataclass
class EndpointListOptions(ListOptions):
    order: t.Optional[Ordering] = None


@dataclass
class EndpointStatsOptions:
    since: t.Optional[datetime] = None
    until: t.Optional[datetime] = None


@dataclass
class IntegrationListOptions(ListOptions):
    pass


class BackgroundTaskListOptions(ListOptions):
    status: t.Optional[BackgroundTaskStatus] = None
    task: t.Optional[BackgroundTaskType] = None


@dataclass
class MessageAttemptListOptions(ListOptions):
    status: t.Optional[MessageStatus] = None
    event_types: t.Optional[t.List[str]] = None
    before: t.Optional[datetime] = None
    after: t.Optional[datetime] = None
    channel: t.Optional[str] = None
    status_code_class: t.Optional[StatusCodeClass] = None

    def to_dict(self) -> t.Dict[str, t.Any]:
        d = super().to_dict()
        if self.before is not None:
            d["before"] = ensure_tz(self.before)
        if self.after is not None:
            d["after"] = ensure_tz(self.after)
        return d


class ApiBase:
    _client: AuthenticatedClient

    def __init__(self, client: AuthenticatedClient) -> None:
        self._client = client


class AuthenticationAsync(ApiBase):
    async def app_portal_access(
        self,
        app_id: str,
        app_portal_access_in: AppPortalAccessIn,
        options: PostOptions = PostOptions(),
    ) -> AppPortalAccessOut:
        return await v1_authentication_app_portal_access.request_asyncio(
            client=self._client,
            app_id=app_id,
            json_body=app_portal_access_in,
            **options.to_dict(),
        )

    async def dashboard_access(
        self, app_id: str, options: PostOptions = PostOptions()
    ) -> DashboardAccessOut:
        return await v1_authentication_dashboard_access.request_asyncio(
            client=self._client, app_id=app_id, **options.to_dict()
        )

    async def logout(self, options: PostOptions = PostOptions()) -> None:
        return await v1_authentication_logout.request_asyncio(
            client=self._client, **options.to_dict()
        )


class Authentication(ApiBase):
    def app_portal_access(
        self,
        app_id: str,
        app_portal_access_in: AppPortalAccessIn,
        options: PostOptions = PostOptions(),
    ) -> AppPortalAccessOut:
        return v1_authentication_app_portal_access.request_sync(
            client=self._client,
            app_id=app_id,
            json_body=app_portal_access_in,
            **options.to_dict(),
        )

    def dashboard_access(
        self, app_id: str, options: PostOptions = PostOptions()
    ) -> DashboardAccessOut:
        return v1_authentication_dashboard_access.request_sync(
            client=self._client, app_id=app_id, **options.to_dict()
        )

    def logout(self, options: PostOptions = PostOptions()) -> None:
        return v1_authentication_logout.request_sync(
            client=self._client, **options.to_dict()
        )


class ApplicationAsync(ApiBase):
    async def list(
        self, options: ApplicationListOptions = ApplicationListOptions()
    ) -> ListResponseApplicationOut:
        return await v1_application_list.request_asyncio(
            client=self._client, **options.to_dict()
        )

    async def create(
        self, application_in: ApplicationIn, options: PostOptions = PostOptions()
    ) -> ApplicationOut:
        return await v1_application_create.request_asyncio(
            client=self._client, json_body=application_in, **options.to_dict()
        )

    async def get(self, app_id: str) -> ApplicationOut:
        return await v1_application_get.request_asyncio(
            client=self._client, app_id=app_id
        )

    async def get_or_create(
        self, application_in: ApplicationIn, options: PostOptions = PostOptions()
    ) -> ApplicationOut:
        return await v1_application_create.request_asyncio(
            client=self._client,
            json_body=application_in,
            get_if_exists=True,
            **options.to_dict(),
        )

    async def update(
        self, app_id: str, application_in: ApplicationIn
    ) -> ApplicationOut:
        return await v1_application_update.request_asyncio(
            client=self._client, app_id=app_id, json_body=application_in
        )

    async def patch(
        self, app_id: str, application_patch: ApplicationPatch
    ) -> ApplicationOut:
        return await v1_application_patch.request_asyncio(
            client=self._client, app_id=app_id, json_body=application_patch
        )

    async def delete(self, app_id: str) -> None:
        return await v1_application_delete.request_asyncio(
            client=self._client, app_id=app_id
        )


class Application(ApiBase):
    def list(
        self, options: ApplicationListOptions = ApplicationListOptions()
    ) -> ListResponseApplicationOut:
        return v1_application_list.request_sync(
            client=self._client, **options.to_dict()
        )

    def create(
        self, application_in: ApplicationIn, options: PostOptions = PostOptions()
    ) -> ApplicationOut:
        return v1_application_create.request_sync(
            client=self._client, json_body=application_in, **options.to_dict()
        )

    def get(self, app_id: str) -> ApplicationOut:
        return v1_application_get.request_sync(client=self._client, app_id=app_id)

    def get_or_create(
        self, application_in: ApplicationIn, options: PostOptions = PostOptions()
    ) -> ApplicationOut:
        return v1_application_create.request_sync(
            client=self._client,
            json_body=application_in,
            get_if_exists=True,
            **options.to_dict(),
        )

    def update(self, app_id: str, application_in: ApplicationIn) -> ApplicationOut:
        return v1_application_update.request_sync(
            client=self._client, app_id=app_id, json_body=application_in
        )

    def patch(self, app_id: str, application_patch: ApplicationPatch) -> ApplicationOut:
        return v1_application_patch.request_sync(
            client=self._client, app_id=app_id, json_body=application_patch
        )

    def delete(self, app_id: str) -> None:
        return v1_application_delete.request_sync(client=self._client, app_id=app_id)


class EndpointAsync(ApiBase):
    async def list(
        self, app_id: str, options: EndpointListOptions = EndpointListOptions()
    ) -> ListResponseEndpointOut:
        return await v1_endpoint_list.request_asyncio(
            client=self._client,
            app_id=app_id,
            **options.to_dict(),
        )

    async def create(
        self, app_id: str, endpoint_in: EndpointIn, options: PostOptions = PostOptions()
    ) -> EndpointOut:
        return await v1_endpoint_create.request_asyncio(
            client=self._client,
            app_id=app_id,
            json_body=endpoint_in,
            **options.to_dict(),
        )

    async def get(self, app_id: str, endpoint_id: str) -> EndpointOut:
        return await v1_endpoint_get.request_asyncio(
            client=self._client, app_id=app_id, endpoint_id=endpoint_id
        )

    async def update(
        self, app_id: str, endpoint_id: str, endpoint_update: EndpointUpdate
    ) -> EndpointOut:
        return await v1_endpoint_update.request_asyncio(
            client=self._client,
            app_id=app_id,
            endpoint_id=endpoint_id,
            json_body=endpoint_update,
        )

    async def patch(
        self, app_id: str, endpoint_id: str, endpoint_patch: EndpointPatch
    ) -> EndpointOut:
        return await v1_endpoint_patch.request_asyncio(
            client=self._client,
            app_id=app_id,
            endpoint_id=endpoint_id,
            json_body=endpoint_patch,
        )

    async def delete(self, app_id: str, endpoint_id: str) -> None:
        return await v1_endpoint_delete.request_asyncio(
            client=self._client,
            app_id=app_id,
            endpoint_id=endpoint_id,
        )

    async def get_secret(self, app_id: str, endpoint_id: str) -> EndpointSecretOut:
        return await v1_endpoint_get_secret.request_asyncio(
            client=self._client,
            app_id=app_id,
            endpoint_id=endpoint_id,
        )

    async def rotate_secret(
        self,
        app_id: str,
        endpoint_id: str,
        endpoint_secret_rotate_in: EndpointSecretRotateIn,
        options: PostOptions = PostOptions(),
    ) -> None:
        return await v1_endpoint_rotate_secret.request_asyncio(
            client=self._client,
            app_id=app_id,
            endpoint_id=endpoint_id,
            json_body=endpoint_secret_rotate_in,
            **options.to_dict(),
        )

    async def recover(
        self,
        app_id: str,
        endpoint_id: str,
        recover_in: RecoverIn,
        options: PostOptions = PostOptions(),
    ) -> RecoverOut:
        return await v1_endpoint_recover.request_asyncio(
            client=self._client,
            app_id=app_id,
            endpoint_id=endpoint_id,
            json_body=recover_in,
            **options.to_dict(),
        )

    async def get_headers(self, app_id: str, endpoint_id: str) -> EndpointHeadersOut:
        return await v1_endpoint_get_headers.request_asyncio(
            client=self._client,
            app_id=app_id,
            endpoint_id=endpoint_id,
        )

    async def update_headers(
        self, app_id: str, endpoint_id: str, endpoint_headers_in: EndpointHeadersIn
    ) -> None:
        return await v1_endpoint_update_headers.request_asyncio(
            client=self._client,
            app_id=app_id,
            endpoint_id=endpoint_id,
            json_body=endpoint_headers_in,
        )

    async def patch_headers(
        self, app_id: str, endpoint_id: str, endpoint_headers_in: EndpointHeadersPatchIn
    ) -> None:
        return await v1_endpoint_patch_headers.request_asyncio(
            client=self._client,
            app_id=app_id,
            endpoint_id=endpoint_id,
            json_body=endpoint_headers_in,
        )

    async def get_stats(
        self,
        app_id: str,
        endpoint_id: str,
        options: EndpointStatsOptions = EndpointStatsOptions(),
    ) -> EndpointStats:
        return await v1_endpoint_get_stats.request_asyncio(
            client=self._client,
            app_id=app_id,
            endpoint_id=endpoint_id,
            since=ensure_tz(options.since),
            until=ensure_tz(options.until),
        )

    async def replay_missing(
        self,
        app_id: str,
        endpoint_id: str,
        replay_in: ReplayIn,
        options: PostOptions = PostOptions(),
    ) -> ReplayOut:
        return await v1_endpoint_replay.request_asyncio(
            client=self._client,
            app_id=app_id,
            endpoint_id=endpoint_id,
            json_body=replay_in,
            **options.to_dict(),
        )

    async def transformations_get(
        self, app_id: str, endpoint_id: str
    ) -> EndpointTransformationOut:
        return await v1_endpoint_transformation_get.request_asyncio(
            client=self._client, app_id=app_id, endpoint_id=endpoint_id
        )

    async def transformation_partial_update(
        self,
        app_id: str,
        endpoint_id: str,
        endpoint_transformation_in: EndpointTransformationIn,
    ) -> None:
        await v1_endpoint_transformation_partial_update.request_asyncio(
            client=self._client,
            app_id=app_id,
            endpoint_id=endpoint_id,
            json_body=endpoint_transformation_in,
        )

    async def send_example(
        self,
        app_id: str,
        endpoint_id: str,
        event_example_in: EventExampleIn,
        options: PostOptions = PostOptions(),
    ) -> MessageOut:
        return await v1_endpoint_send_example.request_asyncio(
            client=self._client,
            app_id=app_id,
            endpoint_id=endpoint_id,
            json_body=event_example_in,
            **options.to_dict(),
        )


class Endpoint(ApiBase):
    def list(
        self, app_id: str, options: EndpointListOptions = EndpointListOptions()
    ) -> ListResponseEndpointOut:
        return v1_endpoint_list.request_sync(
            client=self._client,
            app_id=app_id,
            **options.to_dict(),
        )

    def create(
        self, app_id: str, endpoint_in: EndpointIn, options: PostOptions = PostOptions()
    ) -> EndpointOut:
        return v1_endpoint_create.request_sync(
            client=self._client,
            app_id=app_id,
            json_body=endpoint_in,
            **options.to_dict(),
        )

    def get(self, app_id: str, endpoint_id: str) -> EndpointOut:
        return v1_endpoint_get.request_sync(
            client=self._client, app_id=app_id, endpoint_id=endpoint_id
        )

    def update(
        self, app_id: str, endpoint_id: str, endpoint_update: EndpointUpdate
    ) -> EndpointOut:
        return v1_endpoint_update.request_sync(
            client=self._client,
            app_id=app_id,
            endpoint_id=endpoint_id,
            json_body=endpoint_update,
        )

    def patch(
        self, app_id: str, endpoint_id: str, endpoint_patch: EndpointPatch
    ) -> EndpointOut:
        return v1_endpoint_patch.request_sync(
            client=self._client,
            app_id=app_id,
            endpoint_id=endpoint_id,
            json_body=endpoint_patch,
        )

    def delete(self, app_id: str, endpoint_id: str) -> None:
        return v1_endpoint_delete.request_sync(
            client=self._client,
            app_id=app_id,
            endpoint_id=endpoint_id,
        )

    def get_secret(self, app_id: str, endpoint_id: str) -> EndpointSecretOut:
        return v1_endpoint_get_secret.request_sync(
            client=self._client,
            app_id=app_id,
            endpoint_id=endpoint_id,
        )

    def rotate_secret(
        self,
        app_id: str,
        endpoint_id: str,
        endpoint_secret_rotate_in: EndpointSecretRotateIn,
        options: PostOptions = PostOptions(),
    ) -> None:
        return v1_endpoint_rotate_secret.request_sync(
            client=self._client,
            app_id=app_id,
            endpoint_id=endpoint_id,
            json_body=endpoint_secret_rotate_in,
            **options.to_dict(),
        )

    def recover(
        self,
        app_id: str,
        endpoint_id: str,
        recover_in: RecoverIn,
        options: PostOptions = PostOptions(),
    ) -> RecoverOut:
        return v1_endpoint_recover.request_sync(
            client=self._client,
            app_id=app_id,
            endpoint_id=endpoint_id,
            json_body=recover_in,
            **options.to_dict(),
        )

    def get_headers(self, app_id: str, endpoint_id: str) -> EndpointHeadersOut:
        return v1_endpoint_get_headers.request_sync(
            client=self._client,
            app_id=app_id,
            endpoint_id=endpoint_id,
        )

    def update_headers(
        self, app_id: str, endpoint_id: str, endpoint_headers_in: EndpointHeadersIn
    ) -> None:
        return v1_endpoint_update_headers.request_sync(
            client=self._client,
            app_id=app_id,
            endpoint_id=endpoint_id,
            json_body=endpoint_headers_in,
        )

    def patch_headers(
        self, app_id: str, endpoint_id: str, endpoint_headers_in: EndpointHeadersPatchIn
    ) -> None:
        return v1_endpoint_patch_headers.request_sync(
            client=self._client,
            app_id=app_id,
            endpoint_id=endpoint_id,
            json_body=endpoint_headers_in,
        )

    def get_stats(
        self,
        app_id: str,
        endpoint_id: str,
        options: EndpointStatsOptions = EndpointStatsOptions(),
    ) -> EndpointStats:
        return v1_endpoint_get_stats.request_sync(
            client=self._client,
            app_id=app_id,
            endpoint_id=endpoint_id,
            since=ensure_tz(options.since),
            until=ensure_tz(options.until),
        )

    def replay_missing(
        self,
        app_id: str,
        endpoint_id: str,
        replay_in: ReplayIn,
        options: PostOptions = PostOptions(),
    ) -> ReplayOut:
        return v1_endpoint_replay.request_sync(
            client=self._client,
            app_id=app_id,
            endpoint_id=endpoint_id,
            json_body=replay_in,
            **options.to_dict(),
        )

    def transformations_get(
        self, app_id: str, endpoint_id: str
    ) -> EndpointTransformationOut:
        return v1_endpoint_transformation_get.request_sync(
            client=self._client, app_id=app_id, endpoint_id=endpoint_id
        )

    def transformation_partial_update(
        self,
        app_id: str,
        endpoint_id: str,
        endpoint_transformation_in: EndpointTransformationIn,
    ) -> None:
        v1_endpoint_transformation_partial_update.request_sync(
            client=self._client,
            app_id=app_id,
            endpoint_id=endpoint_id,
            json_body=endpoint_transformation_in,
        )

    def send_example(
        self,
        app_id: str,
        endpoint_id: str,
        event_example_in: EventExampleIn,
        options: PostOptions = PostOptions(),
    ) -> MessageOut:
        return v1_endpoint_send_example.request_sync(
            client=self._client,
            app_id=app_id,
            endpoint_id=endpoint_id,
            json_body=event_example_in,
            **options.to_dict(),
        )


class EventTypeAsync(ApiBase):
    async def list(
        self, options: EventTypeListOptions = EventTypeListOptions()
    ) -> ListResponseEventTypeOut:
        return await v1_event_type_list.request_asyncio(
            client=self._client,
            **options.to_dict(),
        )

    async def create(
        self, event_type_in: EventTypeIn, options: PostOptions = PostOptions()
    ) -> EventTypeOut:
        return await v1_event_type_create.request_asyncio(
            client=self._client,
            json_body=event_type_in,
            **options.to_dict(),
        )

    async def get(self, event_type_name: str) -> EventTypeOut:
        return await v1_event_type_get.request_asyncio(
            client=self._client,
            event_type_name=event_type_name,
        )

    async def update(
        self, event_type_name: str, event_type_update: EventTypeUpdate
    ) -> EventTypeOut:
        return await v1_event_type_update.request_asyncio(
            client=self._client,
            event_type_name=event_type_name,
            json_body=event_type_update,
        )

    async def patch(
        self, event_type_name: str, event_type_patch: EventTypePatch
    ) -> EventTypeOut:
        return await v1_event_type_patch.request_asyncio(
            client=self._client,
            event_type_name=event_type_name,
            json_body=event_type_patch,
        )

    async def delete(self, event_type_name: str) -> None:
        return await v1_event_type_delete.request_asyncio(
            client=self._client,
            event_type_name=event_type_name,
        )

    async def import_openapi(
        self,
        event_type_import_openapi_in: EventTypeImportOpenApiIn,
        options: PostOptions = PostOptions(),
    ) -> EventTypeImportOpenApiOut:
        return await v1_event_type_import_openapi.request_asyncio(
            client=self._client,
            json_body=event_type_import_openapi_in,
            **options.to_dict(),
        )


class EventType(ApiBase):
    def list(
        self, options: EventTypeListOptions = EventTypeListOptions()
    ) -> ListResponseEventTypeOut:
        return v1_event_type_list.request_sync(
            client=self._client,
            **options.to_dict(),
        )

    def create(
        self, event_type_in: EventTypeIn, options: PostOptions = PostOptions()
    ) -> EventTypeOut:
        return v1_event_type_create.request_sync(
            client=self._client,
            json_body=event_type_in,
            **options.to_dict(),
        )

    def get(self, event_type_name: str) -> EventTypeOut:
        return v1_event_type_get.request_sync(
            client=self._client,
            event_type_name=event_type_name,
        )

    def update(
        self, event_type_name: str, event_type_update: EventTypeUpdate
    ) -> EventTypeOut:
        return v1_event_type_update.request_sync(
            client=self._client,
            event_type_name=event_type_name,
            json_body=event_type_update,
        )

    def patch(
        self, event_type_name: str, event_type_patch: EventTypePatch
    ) -> EventTypeOut:
        return v1_event_type_patch.request_sync(
            client=self._client,
            event_type_name=event_type_name,
            json_body=event_type_patch,
        )

    def delete(self, event_type_name: str) -> None:
        return v1_event_type_delete.request_sync(
            client=self._client,
            event_type_name=event_type_name,
        )

    def import_openapi(
        self,
        event_type_import_openapi_in: EventTypeImportOpenApiIn,
        options: PostOptions = PostOptions(),
    ) -> EventTypeImportOpenApiOut:
        return v1_event_type_import_openapi.request_sync(
            client=self._client,
            json_body=event_type_import_openapi_in,
            **options.to_dict(),
        )


class IntegrationAsync(ApiBase):
    async def list(
        self, app_id: str, options: IntegrationListOptions = IntegrationListOptions()
    ) -> ListResponseIntegrationOut:
        return await v1_integration_list.request_asyncio(
            client=self._client,
            app_id=app_id,
            **options.to_dict(),
        )

    async def create(
        self, app_id: str, integ_in: IntegrationIn, options: PostOptions = PostOptions()
    ) -> IntegrationOut:
        return await v1_integration_create.request_asyncio(
            client=self._client,
            app_id=app_id,
            json_body=integ_in,
            **options.to_dict(),
        )

    async def get(self, app_id: str, integ_id: str) -> IntegrationOut:
        return await v1_integration_get.request_asyncio(
            client=self._client,
            app_id=app_id,
            integ_id=integ_id,
        )

    async def update(
        self, app_id: str, integ_id: str, integ_update: IntegrationUpdate
    ) -> IntegrationOut:
        return await v1_integration_update.request_asyncio(
            client=self._client,
            app_id=app_id,
            integ_id=integ_id,
            json_body=integ_update,
        )

    async def delete(self, app_id: str, integ_id: str) -> None:
        return await v1_integration_delete.request_asyncio(
            client=self._client,
            app_id=app_id,
            integ_id=integ_id,
        )

    async def get_key(self, app_id: str, integ_id: str) -> IntegrationKeyOut:
        return await v1_integration_get_key.request_asyncio(
            client=self._client,
            app_id=app_id,
            integ_id=integ_id,
        )

    async def rotate_key(
        self, app_id: str, integ_id: str, options: PostOptions = PostOptions()
    ) -> IntegrationKeyOut:
        return await v1_integration_rotate_key.request_asyncio(
            client=self._client,
            app_id=app_id,
            integ_id=integ_id,
            **options.to_dict(),
        )


class Integration(ApiBase):
    def list(
        self, app_id: str, options: IntegrationListOptions = IntegrationListOptions()
    ) -> ListResponseIntegrationOut:
        return v1_integration_list.request_sync(
            client=self._client,
            app_id=app_id,
            **options.to_dict(),
        )

    def create(
        self, app_id: str, integ_in: IntegrationIn, options: PostOptions = PostOptions()
    ) -> IntegrationOut:
        return v1_integration_create.request_sync(
            client=self._client,
            app_id=app_id,
            json_body=integ_in,
            **options.to_dict(),
        )

    def get(self, app_id: str, integ_id: str) -> IntegrationOut:
        return v1_integration_get.request_sync(
            client=self._client,
            app_id=app_id,
            integ_id=integ_id,
        )

    def update(
        self, app_id: str, integ_id: str, integ_update: IntegrationUpdate
    ) -> IntegrationOut:
        return v1_integration_update.request_sync(
            client=self._client,
            app_id=app_id,
            integ_id=integ_id,
            json_body=integ_update,
        )

    def delete(self, app_id: str, integ_id: str) -> None:
        return v1_integration_delete.request_sync(
            client=self._client,
            app_id=app_id,
            integ_id=integ_id,
        )

    def get_key(self, app_id: str, integ_id: str) -> IntegrationKeyOut:
        return v1_integration_get_key.request_sync(
            client=self._client,
            app_id=app_id,
            integ_id=integ_id,
        )

    def rotate_key(
        self, app_id: str, integ_id: str, options: PostOptions = PostOptions()
    ) -> IntegrationKeyOut:
        return v1_integration_rotate_key.request_sync(
            client=self._client,
            app_id=app_id,
            integ_id=integ_id,
            **options.to_dict(),
        )


class MessageAsync(ApiBase):
    async def list(
        self, app_id: str, options: MessageListOptions = MessageListOptions()
    ) -> ListResponseMessageOut:
        return await v1_message_list.request_asyncio(
            client=self._client,
            app_id=app_id,
            **options.to_dict(),
        )

    async def create(
        self, app_id: str, message_in: MessageIn, options: PostOptions = PostOptions()
    ) -> MessageOut:
        ret = await v1_message_create.request_asyncio(
            client=self._client,
            app_id=app_id,
            json_body=message_in,
            with_content=False,
            **options.to_dict(),
        )
        ret.payload = message_in.payload
        return ret

    async def get(self, app_id: str, msg_id: str) -> MessageOut:
        return await v1_message_get.request_asyncio(
            client=self._client,
            app_id=app_id,
            msg_id=msg_id,
        )

    async def expunge_content(self, app_id: str, msg_id: str) -> None:
        return await v1_message_expunge_content.request_asyncio(
            client=self._client,
            app_id=app_id,
            msg_id=msg_id,
        )


class Message(ApiBase):
    def list(
        self, app_id: str, options: MessageListOptions = MessageListOptions()
    ) -> ListResponseMessageOut:
        return v1_message_list.request_sync(
            client=self._client,
            app_id=app_id,
            **options.to_dict(),
        )

    def create(
        self, app_id: str, message_in: MessageIn, options: PostOptions = PostOptions()
    ) -> MessageOut:
        ret = v1_message_create.request_sync(
            client=self._client,
            app_id=app_id,
            json_body=message_in,
            with_content=False,
            **options.to_dict(),
        )
        ret.payload = message_in.payload
        return ret

    def get(self, app_id: str, msg_id: str) -> MessageOut:
        return v1_message_get.request_sync(
            client=self._client,
            app_id=app_id,
            msg_id=msg_id,
        )

    def expunge_content(self, app_id: str, msg_id: str) -> None:
        return v1_message_expunge_content.request_sync(
            client=self._client,
            app_id=app_id,
            msg_id=msg_id,
        )


class MessageAttemptAsync(ApiBase):
    async def list_by_msg(
        self,
        app_id: str,
        msg_id: str,
        options: MessageAttemptListOptions = MessageAttemptListOptions(),
    ) -> ListResponseMessageAttemptOut:
        return await v1_message_attempt_list_by_msg.request_asyncio(
            client=self._client, app_id=app_id, msg_id=msg_id, **options.to_dict()
        )

    async def list_by_endpoint(
        self,
        app_id: str,
        endpoint_id: str,
        options: MessageAttemptListOptions = MessageAttemptListOptions(),
    ) -> ListResponseMessageAttemptOut:
        return await v1_message_attempt_list_by_endpoint.request_asyncio(
            client=self._client,
            app_id=app_id,
            endpoint_id=endpoint_id,
            **options.to_dict(),
        )

    async def get(self, app_id: str, msg_id: str, attempt_id: str) -> MessageAttemptOut:
        return await v1_message_attempt_get.request_asyncio(
            client=self._client,
            app_id=app_id,
            msg_id=msg_id,
            attempt_id=attempt_id,
        )

    async def resend(
        self,
        app_id: str,
        msg_id: str,
        endpoint_id: str,
        options: PostOptions = PostOptions(),
    ) -> None:
        return await v1_message_attempt_resend.request_asyncio(
            client=self._client,
            app_id=app_id,
            msg_id=msg_id,
            endpoint_id=endpoint_id,
            **options.to_dict(),
        )

    async def list_attempted_messages(
        self,
        app_id: str,
        endpoint_id: str,
        options: MessageAttemptListOptions = MessageAttemptListOptions(),
    ) -> ListResponseEndpointMessageOut:
        return await v1_message_attempt_list_attempted_messages.request_asyncio(
            client=self._client,
            app_id=app_id,
            endpoint_id=endpoint_id,
            **options.to_dict(),
        )

    async def list_attempted_destinations(
        self,
        app_id: str,
        msg_id: str,
        options: MessageAttemptListOptions = MessageAttemptListOptions(),
    ) -> ListResponseMessageEndpointOut:
        return await v1_message_attempt_list_attempted_destinations.request_asyncio(
            client=self._client,
            app_id=app_id,
            msg_id=msg_id,
            **options.to_dict(),
        )

    async def list_attempts_for_endpoint(
        self,
        app_id: str,
        msg_id: str,
        endpoint_id: str,
        options: MessageAttemptListOptions = MessageAttemptListOptions(),
    ) -> ListResponseMessageAttemptEndpointOut:
        return await v1_message_attempt_list_by_endpoint_deprecated.request_asyncio(
            client=self._client,
            app_id=app_id,
            msg_id=msg_id,
            endpoint_id=endpoint_id,
            **options.to_dict(),
        )

    async def expunge_content(
        self,
        app_id: str,
        msg_id: str,
        attempt_id: str,
    ) -> None:
        return await v1_message_attempt_expunge_content.request_asyncio(
            client=self._client,
            app_id=app_id,
            msg_id=msg_id,
            attempt_id=attempt_id,
        )


class MessageAttempt(ApiBase):
    @deprecated(reason="use list_by_msg or list_by_endpoint instead")
    def list(
        self,
        app_id: str,
        msg_id: str,
        options: MessageAttemptListOptions = MessageAttemptListOptions(),
    ) -> ListResponseMessageAttemptOut:
        return self.list_by_msg(app_id=app_id, msg_id=msg_id, options=options)

    def list_by_msg(
        self,
        app_id: str,
        msg_id: str,
        options: MessageAttemptListOptions = MessageAttemptListOptions(),
    ) -> ListResponseMessageAttemptOut:
        return v1_message_attempt_list_by_msg.request_sync(
            client=self._client, app_id=app_id, msg_id=msg_id, **options.to_dict()
        )

    def list_by_endpoint(
        self,
        app_id: str,
        endpoint_id: str,
        options: MessageAttemptListOptions = MessageAttemptListOptions(),
    ) -> ListResponseMessageAttemptOut:
        return v1_message_attempt_list_by_endpoint.request_sync(
            client=self._client,
            app_id=app_id,
            endpoint_id=endpoint_id,
            **options.to_dict(),
        )

    def get(self, app_id: str, msg_id: str, attempt_id: str) -> MessageAttemptOut:
        return v1_message_attempt_get.request_sync(
            client=self._client,
            app_id=app_id,
            msg_id=msg_id,
            attempt_id=attempt_id,
        )

    def resend(
        self,
        app_id: str,
        msg_id: str,
        endpoint_id: str,
        options: PostOptions = PostOptions(),
    ) -> None:
        return v1_message_attempt_resend.request_sync(
            client=self._client,
            app_id=app_id,
            msg_id=msg_id,
            endpoint_id=endpoint_id,
            **options.to_dict(),
        )

    def list_attempted_messages(
        self,
        app_id: str,
        endpoint_id: str,
        options: MessageAttemptListOptions = MessageAttemptListOptions(),
    ) -> ListResponseEndpointMessageOut:
        return v1_message_attempt_list_attempted_messages.request_sync(
            client=self._client,
            app_id=app_id,
            endpoint_id=endpoint_id,
            **options.to_dict(),
        )

    def list_attempted_destinations(
        self,
        app_id: str,
        msg_id: str,
        options: MessageAttemptListOptions = MessageAttemptListOptions(),
    ) -> ListResponseMessageEndpointOut:
        return v1_message_attempt_list_attempted_destinations.request_sync(
            client=self._client,
            app_id=app_id,
            msg_id=msg_id,
            **options.to_dict(),
        )

    def list_attempts_for_endpoint(
        self,
        app_id: str,
        msg_id: str,
        endpoint_id: str,
        options: MessageAttemptListOptions = MessageAttemptListOptions(),
    ) -> ListResponseMessageAttemptEndpointOut:
        return v1_message_attempt_list_by_endpoint_deprecated.request_sync(
            client=self._client,
            app_id=app_id,
            msg_id=msg_id,
            endpoint_id=endpoint_id,
            **options.to_dict(),
        )

    def expunge_content(
        self,
        app_id: str,
        msg_id: str,
        attempt_id: str,
    ) -> None:
        return v1_message_attempt_expunge_content.request_sync(
            client=self._client,
            app_id=app_id,
            msg_id=msg_id,
            attempt_id=attempt_id,
        )


class BackgroundTaskAsync(ApiBase):
    async def list(
        self, options: BackgroundTaskListOptions = BackgroundTaskListOptions()
    ) -> ListResponseBackgroundTaskOut:
        return await list_background_tasks.request_asyncio(
            client=self._client, **options.to_dict()
        )

    async def get(self, task_id: str) -> BackgroundTaskOut:
        return await get_background_task.request_asyncio(
            client=self._client, task_id=task_id
        )


class BackgroundTask(ApiBase):
    def list(
        self, options: BackgroundTaskListOptions = BackgroundTaskListOptions()
    ) -> ListResponseBackgroundTaskOut:
        return list_background_tasks.request_sync(
            client=self._client, **options.to_dict()
        )

    def get(self, task_id: str) -> BackgroundTaskOut:
        return get_background_task.request_sync(client=self._client, task_id=task_id)


class StatisticsAsync(ApiBase):
    async def aggregate_app_stats(
        self, app_usage_stats_in: AppUsageStatsIn, options: PostOptions = PostOptions()
    ) -> AppUsageStatsOut:
        return await v1_statistics_aggregate_app_stats.request_asyncio(
            client=self._client,
            json_body=app_usage_stats_in,
            **options.to_dict(),
        )

    async def aggregate_event_types(self, task_id: str) -> AggregateEventTypesOut:
        return await v1_statistics_aggregate_event_types.request_asyncio(
            client=self._client
        )


class Statistics(ApiBase):
    def aggregate_app_stats(
        self, app_usage_stats_in: AppUsageStatsIn, options: PostOptions = PostOptions()
    ) -> AppUsageStatsOut:
        return v1_statistics_aggregate_app_stats.request_sync(
            client=self._client,
            json_body=app_usage_stats_in,
            **options.to_dict(),
        )

    def aggregate_event_types(self, task_id: str) -> AggregateEventTypesOut:
        return v1_statistics_aggregate_event_types.request_sync(client=self._client)


class ClientBase:
    _client: AuthenticatedClient

    def __init__(self, auth_token: str, options: SvixOptions = SvixOptions()) -> None:
        from . import __version__

        if len(options.retry_schedule) > 5:
            raise ValueError("number of retries must not exceed 5")

        regional_url = None
        region = auth_token.split(".")[-1]
        if region == "us":
            regional_url = "https://api.us.svix.com"
        elif region == "eu":
            regional_url = "https://api.eu.svix.com"
        elif region == "in":
            regional_url = "https://api.in.svix.com"

        host = options.server_url or regional_url or DEFAULT_SERVER_URL
        client = AuthenticatedClient(
            base_url=host,
            token=auth_token,
            headers={"user-agent": f"svix-libs/{__version__}/python"},
            verify_ssl=True,
            retry_schedule=options.retry_schedule,
            timeout=options.timeout,
            follow_redirects=False,
            raise_on_unexpected_status=True,
        )
        self._client = client


class SvixAsync(ClientBase):
    @property
    def authentication(self) -> AuthenticationAsync:
        return AuthenticationAsync(self._client)

    @property
    def application(self) -> ApplicationAsync:
        return ApplicationAsync(self._client)

    @property
    def endpoint(self) -> EndpointAsync:
        return EndpointAsync(self._client)

    @property
    def event_type(self) -> EventTypeAsync:
        return EventTypeAsync(self._client)

    @property
    def integration(self) -> IntegrationAsync:
        return IntegrationAsync(self._client)

    @property
    def message(self) -> MessageAsync:
        return MessageAsync(self._client)

    @property
    def message_attempt(self) -> MessageAttemptAsync:
        return MessageAttemptAsync(self._client)

    @property
    def statistics(self) -> StatisticsAsync:
        return StatisticsAsync(self._client)


class Svix(ClientBase):
    @property
    def authentication(self) -> Authentication:
        return Authentication(self._client)

    @property
    def application(self) -> Application:
        return Application(self._client)

    @property
    def endpoint(self) -> Endpoint:
        return Endpoint(self._client)

    @property
    def event_type(self) -> EventType:
        return EventType(self._client)

    @property
    def integration(self) -> Integration:
        return Integration(self._client)

    @property
    def message(self) -> Message:
        return Message(self._client)

    @property
    def message_attempt(self) -> MessageAttempt:
        return MessageAttempt(self._client)

    @property
    def statistics(self) -> Statistics:
        return Statistics(self._client)


__all__ = [
    "ApplicationIn",
    "ApplicationOut",
    "ApplicationPatch",
    "ListResponseApplicationOut",
    "DashboardAccessOut",
    "EndpointHeadersIn",
    "EndpointHeadersPatchIn",
    "EndpointHeadersOut",
    "EndpointIn",
    "EndpointOut",
    "EndpointPatch",
    "EndpointSecretOut",
    "EndpointSecretRotateIn",
    "ListResponseEndpointOut",
    "EventTypeImportOpenApiIn",
    "EventTypeImportOpenApiOut",
    "EventTypeIn",
    "EventTypeOut",
    "EventTypePatch",
    "EventTypeUpdate",
    "ListResponseEventTypeOut",
    "ListResponseMessageOut",
    "MessageIn",
    "MessageInPayload",
    "MessageOut",
    "MessageOutPayload",
    "EndpointMessageOutPayload",
    "ListResponseMessageAttemptOut",
    "ListResponseEndpointMessageOut",
    "ListResponseMessageEndpointOut",
    "ListResponseMessageAttemptEndpointOut",
    "MessageAttemptOut",
    "MessageStatus",
    "SvixOptions",
    "ApplicationListOptions",
    "EventTypeListOptions",
    "EndpointListOptions",
    "MessageAttemptListOptions",
    "RecoverIn",
    "StatusCodeClass",
    "Svix",
    "SvixAsync",
    "IntegrationKeyOut",
    "IntegrationIn",
    "IntegrationOut",
    "IntegrationUpdate",
    "ListResponseIntegrationOut",
]



# TODO - remove this special case when we fix the generated code for empty openapi structs
from typing import Any, Dict, TypeVar

T = TypeVar("T", bound="EndpointHeadersInHeaders")

EndpointHeadersInHeaders = Dict[str, Any]



# TODO - remove this special case when we fix the generated code for empty openapi structs
import datetime
from typing import Any, Dict, List, Type, TypeVar, Union

import attr
from dateutil.parser import isoparse

from ..types import UNSET, Unset

T = TypeVar("T", bound="StreamOut")


@attr.s(auto_attribs=True)
class StreamOut:
    """
    Attributes:
        created_at (datetime.datetime):
        id (str):
        updated_at (datetime.datetime):
        description (Union[Unset, None, str]):
        uid (Union[Unset, str]):
    """

    created_at: datetime.datetime
    id: str
    updated_at: datetime.datetime
    description: Union[Unset, None, str] = UNSET
    uid: Union[Unset, str] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        created_at = self.created_at.isoformat()

        id = self.id
        updated_at = self.updated_at.isoformat()

        description = self.description
        uid = self.uid

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "createdAt": created_at,
                "id": id,
                "updatedAt": updated_at,
            }
        )
        if description is not UNSET:
            field_dict["description"] = description
        if uid is not UNSET:
            field_dict["uid"] = uid

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        created_at = isoparse(d.pop("createdAt"))

        id = d.pop("id")

        updated_at = isoparse(d.pop("updatedAt"))

        description = d.pop("description", UNSET)

        uid = d.pop("uid", UNSET)

        stream_out = cls(
            created_at=created_at,
            id=id,
            updated_at=updated_at,
            description=description,
            uid=uid,
        )

        stream_out.additional_properties = d
        return stream_out

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



# TODO - remove this special case when we fix the generated code for empty openapi structs
from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union, cast

import attr

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.message_attempt_headers_out_sent_headers import MessageAttemptHeadersOutSentHeaders


T = TypeVar("T", bound="MessageAttemptHeadersOut")


@attr.s(auto_attribs=True)
class MessageAttemptHeadersOut:
    """
    Attributes:
        sensitive (List[str]):
        sent_headers (MessageAttemptHeadersOutSentHeaders):
        response_headers (Union[Unset, None, List[List[str]]]):
    """

    sensitive: List[str]
    sent_headers: "MessageAttemptHeadersOutSentHeaders"
    response_headers: Union[Unset, None, List[List[str]]] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        sensitive = self.sensitive

        sent_headers = self.sent_headers
        response_headers: Union[Unset, None, List[List[str]]] = UNSET
        if not isinstance(self.response_headers, Unset):
            if self.response_headers is None:
                response_headers = None
            else:
                response_headers = []
                for response_headers_item_data in self.response_headers:
                    response_headers_item = response_headers_item_data

                    response_headers.append(response_headers_item)

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "sensitive": sensitive,
                "sentHeaders": sent_headers,
            }
        )
        if response_headers is not UNSET:
            field_dict["responseHeaders"] = response_headers

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        sensitive = cast(List[str], d.pop("sensitive"))

        sent_headers = d.pop("sentHeaders")

        response_headers = []
        _response_headers = d.pop("responseHeaders", UNSET)
        for response_headers_item_data in _response_headers or []:
            response_headers_item = cast(List[str], response_headers_item_data)

            response_headers.append(response_headers_item)

        message_attempt_headers_out = cls(
            sensitive=sensitive,
            sent_headers=sent_headers,
            response_headers=response_headers,
        )

        message_attempt_headers_out.additional_properties = d
        return message_attempt_headers_out

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



# TODO - remove this special case when we fix the generated code for empty openapi structs
from typing import Any, Dict, List, Type, TypeVar, Union

import attr

from ..types import UNSET, Unset

T = TypeVar("T", bound="EndpointMtlsConfigIn")


@attr.s(auto_attribs=True)
class EndpointMtlsConfigIn:
    """
    Attributes:
        identity (str): A PEM encoded private key and X509 certificate to identify the webhook sender.
        server_ca_cert (Union[Unset, None, str]): A PEM encoded X509 certificate used to verify the webhook receiver's
            certificate.
    """

    identity: str
    server_ca_cert: Union[Unset, None, str] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        identity = self.identity
        server_ca_cert = self.server_ca_cert

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "identity": identity,
            }
        )
        if server_ca_cert is not UNSET:
            field_dict["serverCaCert"] = server_ca_cert

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        identity = d.pop("identity")

        server_ca_cert = d.pop("serverCaCert", UNSET)

        endpoint_mtls_config_in = cls(
            identity=identity,
            server_ca_cert=server_ca_cert,
        )

        endpoint_mtls_config_in.additional_properties = d
        return endpoint_mtls_config_in

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



# TODO - remove this special case when we fix the generated code for empty openapi structs
import datetime
from typing import Any, Dict, List, Type, TypeVar, Union, cast

import attr
from dateutil.parser import isoparse

from ..types import UNSET, Unset

T = TypeVar("T", bound="AppUsageStatsIn")


@attr.s(auto_attribs=True)
class AppUsageStatsIn:
    """
    Attributes:
        since (datetime.datetime):
        until (datetime.datetime):
        app_ids (Union[Unset, None, List[str]]):
    """

    since: datetime.datetime
    until: datetime.datetime
    app_ids: Union[Unset, None, List[str]] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        since = self.since.isoformat()

        until = self.until.isoformat()

        app_ids: Union[Unset, None, List[str]] = UNSET
        if not isinstance(self.app_ids, Unset):
            if self.app_ids is None:
                app_ids = None
            else:
                app_ids = self.app_ids

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "since": since,
                "until": until,
            }
        )
        if app_ids is not UNSET:
            field_dict["appIds"] = app_ids

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        since = isoparse(d.pop("since"))

        until = isoparse(d.pop("until"))

        app_ids = cast(List[str], d.pop("appIds", UNSET))

        app_usage_stats_in = cls(
            since=since,
            until=until,
            app_ids=app_ids,
        )

        app_usage_stats_in.additional_properties = d
        return app_usage_stats_in

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from enum import Enum


class KafkaSecurityProtocolType(str, Enum):
    PLAINTEXT = "plaintext"
    SASL_SSL = "sasl-ssl"
    SSL = "ssl"

    def __str__(self) -> str:
        return str(self.value)



import ssl
from typing import Dict, List, Union

import attr


@attr.s(auto_attribs=True)
class Client:
    """A class for keeping track of data related to the API

    Attributes:
        base_url: The base URL for the API, all requests are made to a relative path to this URL
        cookies: A dictionary of cookies to be sent with every request
        headers: A dictionary of headers to be sent with every request
        timeout: The maximum amount of a time in seconds a request can take. API functions will raise
            httpx.TimeoutException if this is exceeded.
        verify_ssl: Whether or not to verify the SSL certificate of the API server. This should be True in production,
            but can be set to False for testing purposes.
        raise_on_unexpected_status: Whether or not to raise an errors.UnexpectedStatus if the API returns a
            status code that was not documented in the source OpenAPI document.
        follow_redirects: Whether or not to follow redirects. Default value is False.
    """

    base_url: str
    cookies: Dict[str, str] = attr.ib(factory=dict, kw_only=True)
    headers: Dict[str, str] = attr.ib(factory=dict, kw_only=True)
    retry_schedule: List[float]
    timeout: float = attr.ib(5.0, kw_only=True)
    verify_ssl: Union[str, bool, ssl.SSLContext] = attr.ib(True, kw_only=True)
    raise_on_unexpected_status: bool = attr.ib(False, kw_only=True)
    follow_redirects: bool = attr.ib(False, kw_only=True)

    def get_headers(self) -> Dict[str, str]:
        """Get headers to be used in all endpoints"""
        return {**self.headers}

    def with_headers(self, headers: Dict[str, str]) -> "Client":
        """Get a new client matching this one with additional headers"""
        return attr.evolve(self, headers={**self.headers, **headers})

    def get_cookies(self) -> Dict[str, str]:
        return {**self.cookies}

    def with_cookies(self, cookies: Dict[str, str]) -> "Client":
        """Get a new client matching this one with additional cookies"""
        return attr.evolve(self, cookies={**self.cookies, **cookies})

    def get_timeout(self) -> float:
        return self.timeout

    def with_timeout(self, timeout: float) -> "Client":
        """Get a new client matching this one with a new timeout (in seconds)"""
        return attr.evolve(self, timeout=timeout)


@attr.s(auto_attribs=True)
class AuthenticatedClient(Client):
    """A Client which has been authenticated for use on secured endpoints"""

    token: str
    prefix: str = "Bearer"
    auth_header_name: str = "Authorization"

    def get_headers(self) -> Dict[str, str]:
        """Get headers to be used in authenticated endpoints"""
        auth_header_value = f"{self.prefix} {self.token}" if self.prefix else self.token
        return {self.auth_header_name: auth_header_value, **self.headers}



# TODO - remove this special case when we fix the generated code for empty openapi structs
from typing import Any, Dict, List, Type, TypeVar

import attr

from ..models.background_task_status import BackgroundTaskStatus
from ..models.background_task_type import BackgroundTaskType

T = TypeVar("T", bound="RecoverOut")


@attr.s(auto_attribs=True)
class RecoverOut:
    """
    Attributes:
        id (str):
        status (BackgroundTaskStatus):
        task (BackgroundTaskType):
    """

    id: str
    status: BackgroundTaskStatus
    task: BackgroundTaskType
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        id = self.id
        status = self.status.value

        task = self.task.value

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "id": id,
                "status": status,
                "task": task,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        id = d.pop("id")

        status = BackgroundTaskStatus(d.pop("status"))

        task = BackgroundTaskType(d.pop("task"))

        recover_out = cls(
            id=id,
            status=status,
            task=task,
        )

        recover_out.additional_properties = d
        return recover_out

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



# TODO - remove this special case when we fix the generated code for empty openapi structs
from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union, cast

import attr

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.event_type_from_open_api import EventTypeFromOpenApi


T = TypeVar("T", bound="EventTypeImportOpenApiOutData")


@attr.s(auto_attribs=True)
class EventTypeImportOpenApiOutData:
    """
    Attributes:
        modified (List[str]):
        to_modify (Union[Unset, None, List['EventTypeFromOpenApi']]):
    """

    modified: List[str]
    to_modify: Union[Unset, None, List["EventTypeFromOpenApi"]] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        modified = self.modified

        to_modify: Union[Unset, None, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.to_modify, Unset):
            if self.to_modify is None:
                to_modify = None
            else:
                to_modify = []
                for to_modify_item_data in self.to_modify:
                    to_modify_item = to_modify_item_data.to_dict()

                    to_modify.append(to_modify_item)

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "modified": modified,
            }
        )
        if to_modify is not UNSET:
            field_dict["to_modify"] = to_modify

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.event_type_from_open_api import EventTypeFromOpenApi

        d = src_dict.copy()
        modified = cast(List[str], d.pop("modified"))

        to_modify = []
        _to_modify = d.pop("to_modify", UNSET)
        for to_modify_item_data in _to_modify or []:
            to_modify_item = EventTypeFromOpenApi.from_dict(to_modify_item_data)

            to_modify.append(to_modify_item)

        event_type_import_open_api_out_data = cls(
            modified=modified,
            to_modify=to_modify,
        )

        event_type_import_open_api_out_data.additional_properties = d
        return event_type_import_open_api_out_data

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



# TODO - remove this special case when we fix the generated code for empty openapi structs
from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar

import attr

from ..models.endpoint_updated_event_type import EndpointUpdatedEventType

if TYPE_CHECKING:
    from ..models.endpoint_updated_event_data import EndpointUpdatedEventData


T = TypeVar("T", bound="EndpointUpdatedEvent")


@attr.s(auto_attribs=True)
class EndpointUpdatedEvent:
    """Sent when an endpoint is updated.

    Attributes:
        data (EndpointUpdatedEventData): Sent when an endpoint is created, updated, or deleted
        type (EndpointUpdatedEventType):  Default: EndpointUpdatedEventType.ENDPOINT_UPDATED.
    """

    data: "EndpointUpdatedEventData"
    type: EndpointUpdatedEventType = EndpointUpdatedEventType.ENDPOINT_UPDATED
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        data = self.data.to_dict()

        type = self.type.value

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "data": data,
                "type": type,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.endpoint_updated_event_data import EndpointUpdatedEventData

        d = src_dict.copy()
        data = EndpointUpdatedEventData.from_dict(d.pop("data"))

        type = EndpointUpdatedEventType(d.pop("type"))

        endpoint_updated_event = cls(
            data=data,
            type=type,
        )

        endpoint_updated_event.additional_properties = d
        return endpoint_updated_event

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



# TODO - remove this special case when we fix the generated code for empty openapi structs
from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union

import attr

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.event_type_import_open_api_in_spec import EventTypeImportOpenApiInSpec


T = TypeVar("T", bound="EventTypeImportOpenApiIn")


@attr.s(auto_attribs=True)
class EventTypeImportOpenApiIn:
    """Import a list of event types from webhooks defined in an OpenAPI spec.

    The OpenAPI spec can be specified as either `spec` given the spec as a JSON object, or as `specRaw` (a `string`)
    which will be parsed as YAML or JSON by the server. Sending neither or both is invalid, resulting in a `400` **Bad
    Request**.

        Attributes:
            dry_run (Union[Unset, bool]): If `true`, return the event types that would be modified without actually
                modifying them.
            spec (Union[Unset, None, EventTypeImportOpenApiInSpec]): A pre-parsed JSON spec. Example: {'info': {'title':
                'Webhook Example', 'version': '1.0.0'}, 'openapi': '3.1.0', 'webhooks': {'pet.new': {'post': {'requestBody':
                {'content': {'application/json': {'schema': {'properties': {'id': {'format': 'int64', 'type': 'integer'},
                'name': {'type': 'string'}, 'tag': {'type': 'string'}}, 'required': ['id', 'name']}}}, 'description':
                'Information about a new pet in the system'}, 'responses': {'200': {'description': 'Return a 200 status to
                indicate that the data was received successfully'}}}}}}.
            spec_raw (Union[Unset, None, str]): A string, parsed by the server as YAML or JSON. Example: {'info': {'title':
                'Webhook Example', 'version': '1.0.0'}, 'openapi': '3.1.0', 'webhooks': {'pet.new': {'post': {'requestBody':
                {'content': {'application/json': {'schema': {'properties': {'id': {'format': 'int64', 'type': 'integer'},
                'name': {'type': 'string'}, 'tag': {'type': 'string'}}, 'required': ['id', 'name']}}}, 'description':
                'Information about a new pet in the system'}, 'responses': {'200': {'description': 'Return a 200 status to
                indicate that the data was received successfully'}}}}}}.
    """

    dry_run: Union[Unset, bool] = False
    spec: Union[Unset, None, "EventTypeImportOpenApiInSpec"] = UNSET
    spec_raw: Union[Unset, None, str] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        dry_run = self.dry_run
        spec = self.spec
        spec_raw = self.spec_raw

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if dry_run is not UNSET:
            field_dict["dry_run"] = dry_run
        if spec is not UNSET:
            field_dict["spec"] = spec
        if spec_raw is not UNSET:
            field_dict["specRaw"] = spec_raw

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        dry_run = d.pop("dry_run", UNSET)

        spec = d.pop("spec", UNSET)

        spec_raw = d.pop("specRaw", UNSET)

        event_type_import_open_api_in = cls(
            dry_run=dry_run,
            spec=spec,
            spec_raw=spec_raw,
        )

        event_type_import_open_api_in.additional_properties = d
        return event_type_import_open_api_in

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from enum import Enum


class Oauth2AuthMethodIn(str, Enum):
    CLIENTSECRETBASIC = "clientSecretBasic"
    CLIENTSECRETJWT = "clientSecretJwt"
    CLIENTSECRETPOST = "clientSecretPost"

    def __str__(self) -> str:
        return str(self.value)



# TODO - remove this special case when we fix the generated code for empty openapi structs
from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union, cast

import attr

from ..models.oauth_2_auth_method_in import Oauth2AuthMethodIn
from ..models.oauth_2_grant_type_in import Oauth2GrantTypeIn
from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.client_secret_jwt_params_in import ClientSecretJwtParamsIn
    from ..models.endpoint_oauth_config_in_extra_params import EndpointOauthConfigInExtraParams


T = TypeVar("T", bound="EndpointOauthConfigIn")


@attr.s(auto_attribs=True)
class EndpointOauthConfigIn:
    """
    Attributes:
        auth_method (Oauth2AuthMethodIn): The method used for authenticating to the OAuth authorization server.

            `clientSecretJwt` will construct a JWT used for authentication with the oauth authorization server. This method
            is less commonly used and may not be supported by all oauth providers. `clientSecretBasic` will authenticate to
            the oauth authorization server using an `Authorization` header with the client secret as the value. This is the
            most common means of authentication. `clientSecretPost` will authenticate to the oauth authorization server by
            passing the client secret in a `client_secret` field in the request body. This method may not be supported by
            all oauth providers, and in general `clientSecretBasic` should be preferred.
        client_id (str): The client ID. Required for all authentication types.
        grant_type (Oauth2GrantTypeIn):
        token_url (str): The URL of the authorization server.
        client_secret (Union[Unset, None, str]): Optional client secret. This is only used for `clientSecretBasic` and
            `clientSecretPost`.

            For `clientSecretBasic`, the secret will be appended to the `Authorization` header. For `clientSecretPost`, this
            will be added to the body in a `client_secret` parameter.
        extra_params (Union[Unset, None, EndpointOauthConfigInExtraParams]): Extra parameters added to the request body
            as key-value pairs.
        jwt_params (Union[Unset, ClientSecretJwtParamsIn]):
        refresh_token (Union[Unset, None, str]): For `refreshToken` grant type
        scopes (Union[Unset, None, List[str]]): Optional OAuth scopes added to the request body.
    """

    auth_method: Oauth2AuthMethodIn
    client_id: str
    grant_type: Oauth2GrantTypeIn
    token_url: str
    client_secret: Union[Unset, None, str] = UNSET
    extra_params: Union[Unset, None, "EndpointOauthConfigInExtraParams"] = UNSET
    jwt_params: Union[Unset, "ClientSecretJwtParamsIn"] = UNSET
    refresh_token: Union[Unset, None, str] = UNSET
    scopes: Union[Unset, None, List[str]] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        auth_method = self.auth_method.value

        client_id = self.client_id
        grant_type = self.grant_type.value

        token_url = self.token_url
        client_secret = self.client_secret
        extra_params = self.extra_params
        jwt_params: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.jwt_params, Unset):
            jwt_params = self.jwt_params.to_dict()

        refresh_token = self.refresh_token
        scopes: Union[Unset, None, List[str]] = UNSET
        if not isinstance(self.scopes, Unset):
            if self.scopes is None:
                scopes = None
            else:
                scopes = self.scopes

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "authMethod": auth_method,
                "clientId": client_id,
                "grantType": grant_type,
                "tokenUrl": token_url,
            }
        )
        if client_secret is not UNSET:
            field_dict["clientSecret"] = client_secret
        if extra_params is not UNSET:
            field_dict["extraParams"] = extra_params
        if jwt_params is not UNSET:
            field_dict["jwtParams"] = jwt_params
        if refresh_token is not UNSET:
            field_dict["refreshToken"] = refresh_token
        if scopes is not UNSET:
            field_dict["scopes"] = scopes

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.client_secret_jwt_params_in import ClientSecretJwtParamsIn

        d = src_dict.copy()
        auth_method = Oauth2AuthMethodIn(d.pop("authMethod"))

        client_id = d.pop("clientId")

        grant_type = Oauth2GrantTypeIn(d.pop("grantType"))

        token_url = d.pop("tokenUrl")

        client_secret = d.pop("clientSecret", UNSET)

        extra_params = d.pop("extraParams", UNSET)

        _jwt_params = d.pop("jwtParams", UNSET)
        jwt_params: Union[Unset, ClientSecretJwtParamsIn]
        if isinstance(_jwt_params, Unset):
            jwt_params = UNSET
        else:
            jwt_params = ClientSecretJwtParamsIn.from_dict(_jwt_params)

        refresh_token = d.pop("refreshToken", UNSET)

        scopes = cast(List[str], d.pop("scopes", UNSET))

        endpoint_oauth_config_in = cls(
            auth_method=auth_method,
            client_id=client_id,
            grant_type=grant_type,
            token_url=token_url,
            client_secret=client_secret,
            extra_params=extra_params,
            jwt_params=jwt_params,
            refresh_token=refresh_token,
            scopes=scopes,
        )

        endpoint_oauth_config_in.additional_properties = d
        return endpoint_oauth_config_in

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



import asyncio
import random
from http import HTTPStatus
from time import sleep
from typing import Any, Dict

import httpx

from ... import errors
from ...client import AuthenticatedClient, Client
from ...models.http_error import HttpError
from ...models.http_validation_error import HTTPValidationError
from ...models.message_attempt_out import MessageAttemptOut
from ...types import Response


def _get_kwargs(
    app_id: str,
    msg_id: str,
    attempt_id: str,
    *,
    client: AuthenticatedClient,
) -> Dict[str, Any]:
    url = "{}/api/v1/app/{app_id}/msg/{msg_id}/attempt/{attempt_id}".format(
        client.base_url, app_id=app_id, msg_id=msg_id, attempt_id=attempt_id
    )

    headers: Dict[str, str] = client.get_headers()
    cookies: Dict[str, Any] = client.get_cookies()

    return {
        "method": "get",
        "url": url,
        "headers": headers,
        "cookies": cookies,
        "timeout": client.get_timeout(),
        "follow_redirects": client.follow_redirects,
    }


def _parse_response(*, client: Client, response: httpx.Response) -> MessageAttemptOut:
    if response.status_code == HTTPStatus.OK:
        response_200 = MessageAttemptOut.from_dict(response.json())

        return response_200
    if response.status_code == HTTPStatus.BAD_REQUEST:
        raise HttpError.init_exception(response.json(), response.status_code)
    if response.status_code == HTTPStatus.UNAUTHORIZED:
        raise HttpError.init_exception(response.json(), response.status_code)
    if response.status_code == HTTPStatus.FORBIDDEN:
        raise HttpError.init_exception(response.json(), response.status_code)
    if response.status_code == HTTPStatus.NOT_FOUND:
        raise HttpError.init_exception(response.json(), response.status_code)
    if response.status_code == HTTPStatus.CONFLICT:
        raise HttpError.init_exception(response.json(), response.status_code)
    if response.status_code == HTTPStatus.UNPROCESSABLE_ENTITY:
        raise HTTPValidationError.init_exception(response.json(), response.status_code)
    if response.status_code == HTTPStatus.TOO_MANY_REQUESTS:
        raise HttpError.init_exception(response.json(), response.status_code)
    raise errors.UnexpectedStatus(response.status_code, response.content)


def _build_response(*, client: Client, response: httpx.Response) -> Response[MessageAttemptOut]:
    return Response(
        status_code=HTTPStatus(response.status_code),
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(client=client, response=response),
    )


def request_sync_detailed(
    app_id: str,
    msg_id: str,
    attempt_id: str,
    *,
    client: AuthenticatedClient,
) -> Response[MessageAttemptOut]:
    """Get Attempt

     `msg_id`: Use a message id or a message `eventId`

    Args:
        app_id (str): The app's ID or UID Example: unique-app-identifier.
        msg_id (str): The msg's ID or UID Example: unique-msg-identifier.
        attempt_id (str): The attempt's ID Example: atmpt_1srOrx2ZWZBpBUvZwXKQmoEYga2.

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[MessageAttemptOut]
    """

    kwargs = _get_kwargs(
        app_id=app_id,
        msg_id=msg_id,
        attempt_id=attempt_id,
        client=client,
    )

    kwargs["headers"] = {"svix-req-id": f"{random.getrandbits(32)}", **kwargs["headers"]}

    response = httpx.request(
        verify=client.verify_ssl,
        **kwargs,
    )
    for retry_count, sleep_time in enumerate(client.retry_schedule):
        if response.status_code < 500:
            break

        sleep(sleep_time)
        kwargs["headers"]["svix-retry-count"] = str(retry_count)
        response = httpx.request(
            verify=client.verify_ssl,
            **kwargs,
        )

    return _build_response(client=client, response=response)


def request_sync(
    app_id: str,
    msg_id: str,
    attempt_id: str,
    *,
    client: AuthenticatedClient,
) -> MessageAttemptOut:
    """Get Attempt

     `msg_id`: Use a message id or a message `eventId`

    Args:
        app_id (str): The app's ID or UID Example: unique-app-identifier.
        msg_id (str): The msg's ID or UID Example: unique-msg-identifier.
        attempt_id (str): The attempt's ID Example: atmpt_1srOrx2ZWZBpBUvZwXKQmoEYga2.

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        MessageAttemptOut
    """

    return request_sync_detailed(
        app_id=app_id,
        msg_id=msg_id,
        attempt_id=attempt_id,
        client=client,
    ).parsed


async def request_asyncio_detailed(
    app_id: str,
    msg_id: str,
    attempt_id: str,
    *,
    client: AuthenticatedClient,
) -> Response[MessageAttemptOut]:
    """Get Attempt

     `msg_id`: Use a message id or a message `eventId`

    Args:
        app_id (str): The app's ID or UID Example: unique-app-identifier.
        msg_id (str): The msg's ID or UID Example: unique-msg-identifier.
        attempt_id (str): The attempt's ID Example: atmpt_1srOrx2ZWZBpBUvZwXKQmoEYga2.

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[MessageAttemptOut]
    """

    kwargs = _get_kwargs(
        app_id=app_id,
        msg_id=msg_id,
        attempt_id=attempt_id,
        client=client,
    )

    kwargs["headers"] = {"svix-req-id": f"{random.getrandbits(32)}", **kwargs["headers"]}

    async with httpx.AsyncClient(verify=client.verify_ssl) as _client:
        response = await _client.request(**kwargs)

        for retry_count, sleep_time in enumerate(client.retry_schedule):
            if response.status_code < 500:
                break

            await asyncio.sleep(sleep_time)
            kwargs["headers"]["svix-retry-count"] = str(retry_count)
            response = await _client.request(**kwargs)

    return _build_response(client=client, response=response)


async def request_asyncio(
    app_id: str,
    msg_id: str,
    attempt_id: str,
    *,
    client: AuthenticatedClient,
) -> MessageAttemptOut:
    """Get Attempt

     `msg_id`: Use a message id or a message `eventId`

    Args:
        app_id (str): The app's ID or UID Example: unique-app-identifier.
        msg_id (str): The msg's ID or UID Example: unique-msg-identifier.
        attempt_id (str): The attempt's ID Example: atmpt_1srOrx2ZWZBpBUvZwXKQmoEYga2.

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        MessageAttemptOut
    """

    return (
        await request_asyncio_detailed(
            app_id=app_id,
            msg_id=msg_id,
            attempt_id=attempt_id,
            client=client,
        )
    ).parsed



# TODO - remove this special case when we fix the generated code for empty openapi structs
from typing import Any, Dict, List, Type, TypeVar, Union

import attr

from ..types import UNSET, Unset

T = TypeVar("T", bound="CustomColorPalette")


@attr.s(auto_attribs=True)
class CustomColorPalette:
    """
    Attributes:
        background_hover (Union[Unset, None, str]):
        background_primary (Union[Unset, None, str]):
        background_secondary (Union[Unset, None, str]):
        button_primary (Union[Unset, None, str]):
        interactive_accent (Union[Unset, None, str]):
        navigation_accent (Union[Unset, None, str]):
        primary (Union[Unset, None, str]):
        text_danger (Union[Unset, None, str]):
        text_primary (Union[Unset, None, str]):
    """

    background_hover: Union[Unset, None, str] = UNSET
    background_primary: Union[Unset, None, str] = UNSET
    background_secondary: Union[Unset, None, str] = UNSET
    button_primary: Union[Unset, None, str] = UNSET
    interactive_accent: Union[Unset, None, str] = UNSET
    navigation_accent: Union[Unset, None, str] = UNSET
    primary: Union[Unset, None, str] = UNSET
    text_danger: Union[Unset, None, str] = UNSET
    text_primary: Union[Unset, None, str] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        background_hover = self.background_hover
        background_primary = self.background_primary
        background_secondary = self.background_secondary
        button_primary = self.button_primary
        interactive_accent = self.interactive_accent
        navigation_accent = self.navigation_accent
        primary = self.primary
        text_danger = self.text_danger
        text_primary = self.text_primary

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if background_hover is not UNSET:
            field_dict["backgroundHover"] = background_hover
        if background_primary is not UNSET:
            field_dict["backgroundPrimary"] = background_primary
        if background_secondary is not UNSET:
            field_dict["backgroundSecondary"] = background_secondary
        if button_primary is not UNSET:
            field_dict["buttonPrimary"] = button_primary
        if interactive_accent is not UNSET:
            field_dict["interactiveAccent"] = interactive_accent
        if navigation_accent is not UNSET:
            field_dict["navigationAccent"] = navigation_accent
        if primary is not UNSET:
            field_dict["primary"] = primary
        if text_danger is not UNSET:
            field_dict["textDanger"] = text_danger
        if text_primary is not UNSET:
            field_dict["textPrimary"] = text_primary

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        background_hover = d.pop("backgroundHover", UNSET)

        background_primary = d.pop("backgroundPrimary", UNSET)

        background_secondary = d.pop("backgroundSecondary", UNSET)

        button_primary = d.pop("buttonPrimary", UNSET)

        interactive_accent = d.pop("interactiveAccent", UNSET)

        navigation_accent = d.pop("navigationAccent", UNSET)

        primary = d.pop("primary", UNSET)

        text_danger = d.pop("textDanger", UNSET)

        text_primary = d.pop("textPrimary", UNSET)

        custom_color_palette = cls(
            background_hover=background_hover,
            background_primary=background_primary,
            background_secondary=background_secondary,
            button_primary=button_primary,
            interactive_accent=interactive_accent,
            navigation_accent=navigation_accent,
            primary=primary,
            text_danger=text_danger,
            text_primary=text_primary,
        )

        custom_color_palette.additional_properties = d
        return custom_color_palette

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



# TODO - remove this special case when we fix the generated code for empty openapi structs
import datetime
from typing import Any, Dict, List, Type, TypeVar, Union, cast

import attr
from dateutil.parser import isoparse

from ..models.transformation_template_kind import TransformationTemplateKind
from ..types import UNSET, Unset

T = TypeVar("T", bound="TemplateOut")


@attr.s(auto_attribs=True)
class TemplateOut:
    """
    Attributes:
        created_at (datetime.datetime):
        description (str):
        id (str):
        instructions (str):
        kind (TransformationTemplateKind):
        logo (str):
        name (str):
        org_id (str):
        transformation (str):
        updated_at (datetime.datetime):
        feature_flag (Union[Unset, None, str]):  Example: cool-new-feature.
        filter_types (Union[Unset, None, List[str]]):  Example: ['user.signup', 'user.deleted'].
        instructions_link (Union[Unset, None, str]):
    """

    created_at: datetime.datetime
    description: str
    id: str
    instructions: str
    kind: TransformationTemplateKind
    logo: str
    name: str
    org_id: str
    transformation: str
    updated_at: datetime.datetime
    feature_flag: Union[Unset, None, str] = UNSET
    filter_types: Union[Unset, None, List[str]] = UNSET
    instructions_link: Union[Unset, None, str] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        created_at = self.created_at.isoformat()

        description = self.description
        id = self.id
        instructions = self.instructions
        kind = self.kind.value

        logo = self.logo
        name = self.name
        org_id = self.org_id
        transformation = self.transformation
        updated_at = self.updated_at.isoformat()

        feature_flag = self.feature_flag
        filter_types: Union[Unset, None, List[str]] = UNSET
        if not isinstance(self.filter_types, Unset):
            if self.filter_types is None:
                filter_types = None
            else:
                filter_types = self.filter_types

        instructions_link = self.instructions_link

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "createdAt": created_at,
                "description": description,
                "id": id,
                "instructions": instructions,
                "kind": kind,
                "logo": logo,
                "name": name,
                "orgId": org_id,
                "transformation": transformation,
                "updatedAt": updated_at,
            }
        )
        if feature_flag is not UNSET:
            field_dict["featureFlag"] = feature_flag
        if filter_types is not UNSET:
            field_dict["filterTypes"] = filter_types
        if instructions_link is not UNSET:
            field_dict["instructionsLink"] = instructions_link

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        created_at = isoparse(d.pop("createdAt"))

        description = d.pop("description")

        id = d.pop("id")

        instructions = d.pop("instructions")

        kind = TransformationTemplateKind(d.pop("kind"))

        logo = d.pop("logo")

        name = d.pop("name")

        org_id = d.pop("orgId")

        transformation = d.pop("transformation")

        updated_at = isoparse(d.pop("updatedAt"))

        feature_flag = d.pop("featureFlag", UNSET)

        filter_types = cast(List[str], d.pop("filterTypes", UNSET))

        instructions_link = d.pop("instructionsLink", UNSET)

        template_out = cls(
            created_at=created_at,
            description=description,
            id=id,
            instructions=instructions,
            kind=kind,
            logo=logo,
            name=name,
            org_id=org_id,
            transformation=transformation,
            updated_at=updated_at,
            feature_flag=feature_flag,
            filter_types=filter_types,
            instructions_link=instructions_link,
        )

        template_out.additional_properties = d
        return template_out

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



# TODO - remove this special case when we fix the generated code for empty openapi structs
from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar

import attr

if TYPE_CHECKING:
    from ..models.completion_message import CompletionMessage


T = TypeVar("T", bound="CompletionChoice")


@attr.s(auto_attribs=True)
class CompletionChoice:
    """
    Attributes:
        finish_reason (str):
        index (int):
        message (CompletionMessage):
    """

    finish_reason: str
    index: int
    message: "CompletionMessage"
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        finish_reason = self.finish_reason
        index = self.index
        message = self.message.to_dict()

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "finish_reason": finish_reason,
                "index": index,
                "message": message,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.completion_message import CompletionMessage

        d = src_dict.copy()
        finish_reason = d.pop("finish_reason")

        index = d.pop("index")

        message = CompletionMessage.from_dict(d.pop("message"))

        completion_choice = cls(
            finish_reason=finish_reason,
            index=index,
            message=message,
        )

        completion_choice.additional_properties = d
        return completion_choice

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



# TODO - remove this special case when we fix the generated code for empty openapi structs
from typing import Any, Dict, List, Type, TypeVar, Union

import attr

from ..models.transformation_http_method import TransformationHttpMethod
from ..types import UNSET, Unset

T = TypeVar("T", bound="EndpointTransformationSimulateOut")


@attr.s(auto_attribs=True)
class EndpointTransformationSimulateOut:
    """
    Attributes:
        payload (str):
        url (str):
        method (Union[Unset, TransformationHttpMethod]):
    """

    payload: str
    url: str
    method: Union[Unset, TransformationHttpMethod] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        payload = self.payload
        url = self.url
        method: Union[Unset, str] = UNSET
        if not isinstance(self.method, Unset):
            method = self.method.value

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "payload": payload,
                "url": url,
            }
        )
        if method is not UNSET:
            field_dict["method"] = method

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        payload = d.pop("payload")

        url = d.pop("url")

        _method = d.pop("method", UNSET)
        method: Union[Unset, TransformationHttpMethod]
        if isinstance(_method, Unset):
            method = UNSET
        else:
            method = TransformationHttpMethod(_method)

        endpoint_transformation_simulate_out = cls(
            payload=payload,
            url=url,
            method=method,
        )

        endpoint_transformation_simulate_out.additional_properties = d
        return endpoint_transformation_simulate_out

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties
